from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadTable(AcadEntity):
    def __init__(self, obj) -> None: ...
    AllowManualHeights: bool
    AllowManualPositions: bool
    BreaksEnabled: bool
    BreakSpacing: float
    Columns: int
    ColumnWidth: float
    Direction: PyGeVector3d
    EnableBreak: bool
    FlowDirection: AcTableDirection
    HasSubSelection: bool
    HeaderSuppressed: bool
    Height: float
    HorzCellMargin: float
    InsertionPoint: PyGePoint3d
    MinimumTableHeight: float
    MinimumTableWidth: float
    RegenerateTableSuppressed: bool
    RepeatBottomLabels: bool
    RowHeight: float
    Rows: int
    StyleName: str
    TableBreakFlowDirection: AcTableFlowDirection
    TableBreakHeight: float
    TableStyleOverrides: AcTableStyleOverrides
    TitleSuppressed: bool
    VertCellMargin: float
    Width: float
    def ClearSubSelection(self) -> None: ...
    def ClearTableStyleOverrides(self, flag: int) -> None: ...
    def Copy(self, flag: int) -> AcadTable: ...
    def CreateContent(self, nRow: int, nCol: int, nIndex: int) -> int: ...
    def DeleteCellContent(self, row: int, col: int) -> None: ...
    def DeleteColumns(self, col: int, cols: int) -> None: ...
    def DeleteContent(self, nRow: int, nCol: int) -> None: ...
    def DeleteRows(self, row: int, Rows: int) -> None: ...
    def EnableMergeAll(self, nRow: int, nCol: int, bEnable: bool) -> None: ...
    def FormatValue(self, row: int, col: int, nOption: AcFormatOption) -> str: ...
    def GenerateLayout(self) -> None: ...
    def GetAlignment(self, rowType: AcRowType) -> AcCellAlignment: ...
    def GetAttachmentPoint(self, row: int, col: int) -> AcAttachmentPoint: ...
    def GetAutoScale(self, row: int, col: int) -> bool: ...
    def GetAutoScale2(self, nRow: int, nCol: int, nContent: int) -> bool: ...
    def GetBackgroundColor(self, rowType: AcRowType) -> AcadAcCmColor: ...
    def GetBackgroundColorNone(self, rowType: AcRowType) -> bool: ...
    def GetBlockAttributeValue(self, row: int, col: int, attdefId: int) -> str: ...
    def GetBlockAttributeValue2(self, nRow: int, nCol: int, nContent: int, blkId: int) -> str: ...
    def GetBlockRotation(self, row: int, col: int) -> float: ...
    def GetBlockScale(self, row: int, col: int) -> float: ...
    def GetBlockTableRecordId(self, row: int, col: int) -> int: ...
    def GetBlockTableRecordId2(self, nRow: int, nCol: int, nContent: int) -> int: ...
    def GetBreakHeight(self, nIndex: int) -> float: ...
    def GetCellAlignment(self, row: int, col: int) -> AcCellAlignment: ...
    def GetCellBackgroundColor(self, row: int, col: int) -> AcadAcCmColor: ...
    def GetCellBackgroundColorNone(self, row: int, col: int) -> bool: ...
    def GetCellContentColor(self, row: int, col: int) -> AcadAcCmColor: ...
    def GetCellDataType(self, row: int, col: int, pDataType: AcValueDataType, pUnitType: AcValueUnitType) -> None: ...
    def GetCellExtents(self, row: int, col: int, bOuterCell: bool) -> None: ...
    def GetCellFormat(self, row: int, col: int) -> str: ...
    def GetCellGridColor(self, row: int, col: int, edge: AcCellEdgeMask) -> AcadAcCmColor: ...
    def GetCellGridLineWeight(self, row: int, col: int, edge: AcCellEdgeMask) -> AcLineWeight: ...
    def GetCellGridVisibility(self, row: int, col: int, edge: AcCellEdgeMask) -> bool: ...
    def GetCellState(self, nRow: int, nCol: int) -> AcCellState: ...
    def GetCellStyle(self, nRow: int, nCol: int) -> str: ...
    def GetCellStyleOverrides(self, row: int, col: int) -> str: ...
    def GetCellTextHeight(self, row: int, col: int) -> float: ...
    def GetCellTextStyle(self, row: int, col: int) -> str: ...
    def GetCellType(self, row: int, col: int) -> AcCellType: ...
    def GetCellValue(self, row: int, col: int) -> Variant: ...
    def GetColumnName(self, nIndex: int) -> str: ...
    def GetColumnWidth(self, col: int) -> float: ...
    def GetContentColor(self, rowType: AcRowType) -> AcadAcCmColor: ...
    def GetCellContentColor2(self, nRow: int, nCol: int, nContent: int) -> AcadAcCmColor: ...
    def GetContentLayout(self, nRow: int, nCol: int) -> AcCellContentLayout: ...
    def GetContentType(self, nRow: int, nCol: int) -> AcCellContentType: ...
    def GetCustomData(self, nRow: int, nCol: int, szKey: str) -> Variant: ...
    def GetDataFormat(self, nRow: int, nCol: int, nContent: int) -> str: ...
    def GetDataType(self, rowType: AcRowType) -> tuple[AcValueDataType, AcValueUnitType]: ...
    def GetDataType2(self, nRow: int, nCol: int, nContent: int) -> tuple[AcValueDataType, AcValueUnitType]: ...
    def GetFieldId(self, row: int, col: int) -> int: ...
    def GetFieldId2(self, nRow: int, nCol: int, nContent: int) -> int: ...
    def GetFormat(self, rowType: AcRowType) -> str: ...
    def GetFormula(self, nRow: int, nCol: int, nContent: int) -> str: ...
    def GetGridColor(self, gridLineType: AcGridLineType, rowType: AcRowType) -> AcadAcCmColor: ...
    def GetGridColor2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType) -> AcadAcCmColor: ...
    def GetGridDoubleLineSpacing(self, nRow: int, nCol: int, nGridLineType: AcGridLineType) -> float: ...
    def GetGridLineStyle(self, nRow: int, nCol: int, nGridLineType: AcGridLineType) -> AcGridLineStyle: ...
    def GetGridLinetype(self, nRow: int, nCol: int, nGridLineType: AcGridLineType) -> int: ...
    def GetGridLineWeight(self, gridLineType: AcGridLineType, rowType: AcRowType) -> AcLineWeight: ...
    def GetGridLineWeight2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType) -> AcLineWeight: ...
    def GetGridVisibility(self, gridLineType: AcGridLineType, rowType: AcRowType) -> bool: ...
    def GetGridVisibility2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType) -> bool: ...
    def GetHasFormula(self, nRow: int, nCol: int, nContent: int) -> bool: ...
    def GetMargin(self, nRow: int, nCol: int, nMargin: AcCellMargin) -> float: ...
    def GetMinimumColumnWidth(self, col: int) -> float: ...
    def GetMinimumRowHeight(self, row: int) -> float: ...
    def GetOverride(self, nRow: int, nCol: int, nContent: int) -> AcCellProperty: ...
    def GetRotation(self, nRow: int, nCol: int, nContent: int) -> float: ...
    def GetRowHeight(self, row: int) -> float: ...
    def GetRowType(self, row: int) -> AcRowType: ...
    def GetScale(self, nRow: int, nCol: int, nContent: int) -> float: ...
    def GetSubSelection(self) -> tuple: ...
    def GetText(self, row: int, col: int) -> str: ...
    def GetTextHeight(self, rowType: AcRowType) -> float: ...
    def GetTextHeight2(self, nRow: int, nCol: int, nContent: int) -> float: ...
    def GetTextRotation(self, row: int, col: int) -> AcRotationAngle: ...
    def GetTextString(self, nRow: int, nCol: int, nContent: int) -> str: ...
    def GetTextStyle(self, rowTypes: AcRowType) -> str: ...
    def GetTextStyle2(self, nRow: int, nCol: int, nContent: int) -> str: ...
    def GetValue(self, nRow: int, nCol: int, nContent: int) -> Variant: ...
    def HitTest(self, wpt: PyGePoint3d, wviewVec: PyGeVector3d) -> vIntegerArray: ...
    def InsertColumns(self, col: int, Width: float, cols: int) -> None: ...
    def InsertColumnsAndInherit(self, col: int, nInheritFrom: int, nNumCols: int) -> None: ...
    def InsertRows(self, row: int, Height: float, Rows: int) -> None: ...
    def InsertRowsAndInherit(self, nIndex: int, nInheritFrom: int, nNumRows: int) -> None: ...
    def IsContentEditable(self, nRow: int, nCol: int) -> bool: ...
    def IsEmpty(self, nRow: int, nCol: int) -> bool: ...
    def IsFormatEditable(self, nRow: int, nCol: int) -> bool: ...
    def IsMergeAllEnabled(self, nRow: int, nCol: int) -> bool: ...
    def IsMergedCell(self, row: int, col: int, minRow: int, maxRow: int, minCol: int, maxCol: int) -> bool: ...
    def MergeCells(self, minRow: int, maxRow: int, minCol: int, maxCol: int) -> None: ...
    def MoveContent(self, nRow: int, nCol: int, nFromIndex: int, nToIndex: int) -> None: ...
    def RecomputeTableBlock(self, bForceUpdate: bool) -> None: ...
    def RemoveAllOverrides(self, nRow: int, nCol: int) -> None: ...
    def ReselectSubRegion(self) -> None: ...
    def ResetCellValue(self, row: int, col: int) -> None: ...
    def Select(self, wpt: PyGePoint3d, wvwVec: PyGeVector3d, wvwxvec: PyGeVector3d, allowOutside: bool) -> tuple[float, float, int, int]: ...
    def SelectSubRegion(self, wpt1: PyGePoint3d, wpt2: PyGePoint3d, wvwVec: PyGeVector3d, wvwxVec: PyGeVector3d, seltype: AcSelectType, bIncludeCurrentSelection: bool) -> tuple[int, int, int, int]: ...
    def SetAlignment(self, rowTypes: AcRowType, cellAlignment: AcCellAlignment) -> None: ...
    def SetAutoScale(self, row: int, col: int, bValue: bool) -> None: ...
    def SetAutoScale2(self, nRow: int, nCol: int, nContent: int, bAutoFit: bool) -> None: ...
    def SetBackgroundColor(self, rowTypes: AcRowType, pColor: AcadAcCmColor) -> None: ...
    def SetBackgroundColorNone(self, rowTypes: AcRowType, bValue: bool) -> None: ...
    def SetBlockAttributeValue(self, row: int, col: int, attdefId: int, StringValue: str) -> None: ...
    def SetBlockAttributeValue2(self, nRow: int, nCol: int, nContent: int, blkId: int, value: str) -> None: ...
    def SetBlockRotation(self, row: int, col: int, blkRotation: float) -> None: ...
    def SetBlockScale(self, row: int, col: int, blkScale: float) -> None: ...
    def SetBlockTableRecordId(self, row: int, col: int, blkId: int, bAutoFit: bool) -> None: ...
    def SetBlockTableRecordId2(self, nRow: int, nCol: int, nContent: int, blkId: int, autoFit: bool) -> None: ...
    def SetBreakHeight(self, nIndex: int, dHeight: float) -> None: ...
    def SetCellAlignment(self, row: int, col: int, cellAlignment: AcCellAlignment) -> None: ...
    def SetCellBackgroundColor(self, row: int, col: int, pColor: AcadAcCmColor) -> None: ...
    def SetCellBackgroundColorNone(self, row: int, col: int, bValue: bool) -> None: ...
    def SetCellContentColor(self, row: int, col: int, pColor: AcadAcCmColor) -> None: ...
    def SetCellDataType(self, row: int, col: int, dataType: AcValueDataType, unitType: AcValueUnitType) -> None: ...
    def SetCellFormat(self, row: int, col: int, pFormat: str) -> None: ...
    def SetCellGridColor(self, row: int, col: int, edges: AcCellEdgeMask, pColor: AcadAcCmColor) -> None: ...
    def SetCellGridLineWeight(self, row: int, col: int, edges: AcCellEdgeMask, Lineweight: AcLineWeight) -> None: ...
    def SetCellGridVisibility(self, row: int, col: int, edges: AcCellEdgeMask, bValue: bool) -> None: ...
    def SetCellState(self, nRow: int, nCol: int, nLock: AcCellState) -> None: ...
    def SetCellStyle(self, nRow: int, nCol: int, szCellStyle: str) -> None: ...
    def SetCellTextHeight(self, row: int, col: int, TextHeight: float) -> None: ...
    def SetCellTextStyle(self, row: int, col: int, bstrName: str) -> None: ...
    def SetCellType(self, row: int, col: int, CellType: AcCellType) -> None: ...
    def SetCellValue(self, row: int, col: int) -> Variant: ...
    def SetCellValueFromText(self, row: int, col: int, val: str, nOption: AcParseOption) -> None: ...
    def SetColumnName(self, nIndex: int, name: str) -> None: ...
    def SetColumnWidth(self, col: int, Width: float) -> None: ...
    def SetContentColor(self, rowTypes: AcRowType, pColor: AcadAcCmColor) -> None: ...
    def SetContentColor2(self, nRow: int, nCol: int, nContent: int, pColor: AcadAcCmColor) -> None: ...
    def SetContentLayout(self, nRow: int, nCol: int, nLayout: AcCellContentLayout) -> None: ...
    def SetCustomData(self, nRow: int, nCol: int, szKey: str, data: Variant) -> None: ...
    def SetDataFormat(self, nRow: int, nCol: int, nContent: int, szFormat: str) -> None: ...
    def SetDataType(self, rowTypes: AcRowType, nDataType: AcValueDataType, nUnitType: AcValueUnitType) -> None: ...
    def SetDataType2(self, nRow: int, nCol: int, nContent: int, dataType: AcValueDataType, unitType: AcValueUnitType) -> None: ...
    def SetFieldId(self, row: int, col: int, fieldId: int) -> None: ...
    def SetFieldId2(self, nRow: int, nCol: int, nContent: int, acDbObjectId: int, nflag: AcCellOption) -> None: ...
    def SetFormat(self, rowTypes: AcRowType, pFormat: str) -> None: ...
    def SetFormula(self, nRow: int, nCol: int, nContent: int, pszFormula: str) -> None: ...
    def SetGridColor(self, gridLineTypes: AcGridLineType, rowTypes: AcRowType, pColor: AcadAcCmColor) -> None: ...
    def SetGridColor2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType, pColor: AcadAcCmColor) -> None: ...
    def SetGridDoubleLineSpacing(self, nRow: int, nCol: int, nGridLineType: AcGridLineType, fSpacing: float) -> None: ...
    def SetGridLineStyle(self, nRow: int, nCol: int, nGridLineType: AcGridLineType, nLineStyle: AcGridLineStyle) -> None: ...
    def SetGridLinetype(self, nRow: int, nCol: int, nGridLineType: AcGridLineType, idLinetype: int) -> None: ...
    def SetGridLineWeight(self, gridLineTypes: AcGridLineType, rowTypes: AcRowType, Lineweight: AcLineWeight) -> None: ...
    def SetGridLineWeight2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType, lineWeight: AcLineWeight) -> None: ...
    def SetGridVisibility(self, gridLineTypes: AcGridLineType, rowTypes: AcRowType, bValue: bool) -> None: ...
    def SetGridVisibility2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType, bVisible: bool) -> None: ...
    def SetMargin(self, nRow: int, nCol: int, nMargins: AcCellMargin, fMargin: float) -> None: ...
    def SetOverride(self, nRow: int, nCol: int, nContent: int, nProp: AcCellProperty) -> None: ...
    def SetRotation(self, nRow: int, nCol: int, nContent: int, pValue: float) -> None: ...
    def SetRowHeight(self, row: int, Height: float) -> None: ...
    def SetScale(self, nRow: int, nCol: int, nContent: int, scale: float) -> None: ...
    def SetSubSelection(self, rowMin: int, rowMax: int, colMin: int, colMax: int) -> None: ...
    def SetText(self, row: int, col: int, pStr: str) -> None: ...
    def SetTextHeight(self, rowTypes: AcRowType, TextHeight: float) -> None: ...
    def SetTextHeight2(self, nRow: int, nCol: int, nContent: int, height: float) -> None: ...
    def SetTextRotation(self, row: int, col: int, TextRotation: AcRotationAngle) -> None: ...
    def SetTextString(self, nRow: int, nCol: int, nContent: int, text: str) -> None: ...
    def SetTextStyle(self, rowTypes: AcRowType, bstrName: str) -> None: ...
    def SetTextStyle2(self, nRow: int, nCol: int, nContent: int, StringStyleName: str) -> None: ...
    def SetToolTip(self, nRow: int, nCol: int, tip: str) -> None: ...
    def SetValue(self, nRow: int, nCol: int, nContent: int, acValue: Variant) -> None: ...
    def SetValueFromText(self, nRow: int, nCol: int, nContent: int, szText: str, nOption: AcParseOption) -> None: ...
    def UnmergeCells(self, minRow: int, maxRow: int, minCol: int, maxCol: int) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadArc(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    ArcLength: float = proxy_property(float,'ArcLength',AccessMode.ReadOnly)
    Area: float = proxy_property(float,'Area',AccessMode.ReadOnly)
    Center: PyGePoint3d = proxy_property('PyGePoint3d','Center',AccessMode.ReadWrite)
    EndAngle: float = proxy_property(float,'EndAngle',AccessMode.ReadWrite)
    EndPoint: PyGePoint3d = proxy_property('PyGePoint3d','EndPoint',AccessMode.ReadOnly)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    Radius: float = proxy_property(float,'Radius',AccessMode.ReadWrite)
    StartAngle: float = proxy_property(float,'StartAngle',AccessMode.ReadWrite)
    StartPoint: PyGePoint3d = proxy_property('PyGePoint3d','StartPoint',AccessMode.ReadWrite)
    Thickness: float = proxy_property(float,'Thickness',AccessMode.ReadWrite)
    TotalAngle: float = proxy_property(float,'TotalAngle',AccessMode.ReadOnly)

    def Copy(self) -> AcadArc:
        return AcadArc(self._obj.Copy())

    def Offset(self, Distance: float) -> vObjectArray:
        return vObjectArray(self._obj.Offset(Distance))


from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadToolbar(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Count: int
    DockStatus: AcToolbarDockStatus
    FloatingRows: int
    Height: int
    HelpString: str
    LargeButtons: bool
    Left: int
    Name: str
    Parent: AppObject
    TagString: str
    Top: int
    Visible: bool
    Width: float
    def AddSeparator(self, Index: int | str) -> AcadToolbarItem: ...
    def AddToolbarButton(self, Index: int | str, Name: str, HelpString: str, Macro: str, FlyoutButton: vBool = None) -> AcadToolbarItem: ...
    def Delete(self) -> None: ...
    def Dock(self, Side: AcToolbarDockStatus) -> None: ...
    def Float(self, Top: int, Left: int, NumberFloatRows: int) -> None: ...
    def Item(self, Index: int | str) -> AcadObject: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadPreferencesSystem(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    BeepOnError: bool = proxy_property(bool,'BeepOnError',AccessMode.ReadWrite)
    DisplayOLEScale: bool = proxy_property(bool,'DisplayOLEScale',AccessMode.ReadWrite)
    EnableStartupDialog: bool = proxy_property(bool,'EnableStartupDialog',AccessMode.ReadWrite)
    LoadAcadLspInAllDocuments: bool = proxy_property(bool,'LoadAcadLspInAllDocuments',AccessMode.ReadWrite)
    ShowWarningMessages: bool = proxy_property(bool,'ShowWarningMessages',AccessMode.ReadWrite)
    SingleDocumentMode: bool = proxy_property(bool,'SingleDocumentMode',AccessMode.ReadWrite)
    StoreSQLIndex: bool = proxy_property(bool,'StoreSQLIndex',AccessMode.ReadWrite)
    TablesReadOnly: bool = proxy_property(bool,'TablesReadOnly',AccessMode.ReadWrite)


from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadXRecord(AcadObject):
    def __init__(self, obj) -> None: ...
    Name: str
    TranslateIDs: bool
    def GetXRecordData(self) -> tuple: ...
    def SetXRecordData(self, XRecordDataType: Variant, XRecordData: Variant) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadToolbar import *

class AcadToolbarItem(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    CommandDisplayName: str = proxy_property(str,'CommandDisplayName',AccessMode.ReadWrite)
    Flyout: AcadToolbar = proxy_property('AcadToolbar','Flyout',AccessMode.ReadOnly)
    HelpString: str = proxy_property(str,'HelpString',AccessMode.ReadWrite)
    Index: int = proxy_property(int,'Index',AccessMode.ReadOnly)
    Macro: str = proxy_property(str,'Macro',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    Parent: AppObject = proxy_property('AppObject','Parent',AccessMode.ReadWrite)
    TagString: str = proxy_property(str,'TagString',AccessMode.ReadWrite)
    Type: AcToolbarItemType = proxy_property('AcToolbarItemType','Type',AccessMode.ReadOnly)

    def AttachToolbarToFlyout(self, MenuGroupName: str, ToolbarName: str) -> None:
        self._obj.AttachToolbarToFlyout(MenuGroupName, ToolbarName)

    def Delete(self) -> None:
        self._obj.Delete()

    def GetBitmaps(self) -> tuple[str]:
        SmallIconName, LargeIconName = self._obj.GetBitmaps()
        return SmallIconName, LargeIconName

    def SetBitmaps(self, SmallIconName: str, LargeIconName: str) -> None:
        self._obj.SetBitmaps(SmallIconName, LargeIconName)



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *



class AcadSectionManager(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    def GetLiveSection(self) -> AcadSection:
        return AcadSection(self._obj.GetLiveSection())

    def GetUniqueSectionName(self) -> str:
        return self._obj.GetUniqueSectionName()



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadTolerance(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    DimensionLineColor: AcColor = proxy_property('AcColor','DimensionLineColor',AccessMode.ReadWrite)
    DirectionVector: PyGeVector3d = proxy_property('PyGeVector3d','DirectionVector',AccessMode.ReadWrite)
    InsertionPoint: PyGePoint3d = proxy_property('PyGePoint3d','InsertionPoint',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    ScaleFactor: float = proxy_property(float,'ScaleFactor',AccessMode.ReadWrite)
    StyleName: str = proxy_property(str,'StyleName',AccessMode.ReadWrite)
    TextColor: AcColor = proxy_property('AcColor','TextColor',AccessMode.ReadWrite)
    TextHeight: float = proxy_property(float,'TextHeight',AccessMode.ReadWrite)
    TextString: str = proxy_property(str,'TextString',AccessMode.ReadWrite)
    TextStyle: AcadTextStyle = proxy_property('AcadTextStyle','TextStyle',AccessMode.ReadWrite)



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadLine(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Angle: float = proxy_property(float,'Angle',AccessMode.ReadOnly)
    Delta: vDoubleArray = proxy_property('vDoubleArray','Delta',AccessMode.ReadOnly)
    EndPoint: PyGePoint3d = proxy_property('PyGePoint3d','EndPoint',AccessMode.ReadWrite)
    Length: float = proxy_property(float,'Length',AccessMode.ReadOnly)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    StartPoint: PyGePoint3d = proxy_property('PyGePoint3d','StartPoint',AccessMode.ReadWrite)
    Thickness: float = proxy_property(float,'Thickness',AccessMode.ReadWrite)

    def Offset(self, Distance: float) -> vObjectArray:
        return vObjectArray(self._obj.Offset(Distance))

    def Copy(self) -> AcadLine:
        return AcadLine(self._obj.Copy())


from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPreferencesDisplay(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    AutoTrackingVecColor: int
    CursorSize: int
    DisplayLayoutTabs: bool
    DisplayScreenMenu: bool
    DisplayScrollBars: bool
    DockedVisibleLines: int
    GraphicsWinLayoutBackgrndColor: int
    GraphicsWinModelBackgrndColor: int
    HistoryLines: int
    ImageFrameHighlight: bool
    LayoutCreateViewport: bool
    LayoutCrosshairColor: int
    LayoutDisplayMargins: bool
    LayoutDisplayPaper: bool
    LayoutDisplayPaperShadow: bool
    MaxAutoCADWindow: bool
    ModelCrosshairColor: int
    ShowRasterImage: bool
    TextFont: str
    TextFontSize: int
    TextFontStyle: AcTextFontStyle
    TextWinBackgrndColor: int
    TextWinTextColor: int
    TrueColorImages: bool
    XRefFadeIntensity: int



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadPoint(AcadEntity):
    def __init__(self, obj) -> None: ...
    Coordinates: PyGePoint3d
    Normal: PyGeVector3d
    Thickness: float
    def Copy(self) -> AcadPoint: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadBlocks import AcadBlocks
from .AcadDictionaries import AcadDictionaries
from .AcadDimStyles import AcadDimStyles
from .AcadGroups import AcadGroups
from .AcadLayers import AcadLayers
from .AcadLayouts import AcadLayouts
from .AcadLineTypes import AcadLineTypes
from .AcadMaterials import AcadMaterials
from .AcadModelSpace import AcadModelSpace
from .AcadPaperSpace import AcadPaperSpace
from .AcadPlotConfigurations import AcadPlotConfigurations
from .AcadDatabasePreferences import AcadDatabasePreferences
from .AcadRegisteredApplications import AcadRegisteredApplications
from .AcadSectionManager import AcadSectionManager
from .AcadSummaryInfo import AcadSummaryInfo
from .AcadTextStyles import AcadTextStyles
from .AcadUCSs import AcadUCSs
from .AcadViewports import AcadViewports
from .AcadViews import AcadViews




class AcadDatabase(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Blocks: AcadBlocks = proxy_property('AcadBlocks','Blocks',AccessMode.ReadOnly)
    Dictionaries: AcadDictionaries = proxy_property('AcadDictionaries','Dictionaries',AccessMode.ReadOnly)
    DimStyles: AcadDimStyles = proxy_property('AcadDimStyles','DimStyles',AccessMode.ReadOnly)
    ElevationModelSpace: float = proxy_property(float,'ElevationModelSpace',AccessMode.ReadWrite)
    ElevationPaperSpace: float = proxy_property(float,'ElevationPaperSpace',AccessMode.ReadWrite)
    Groups: AcadGroups = proxy_property('AcadGroups','Groups',AccessMode.ReadOnly)
    Layers: AcadLayers = proxy_property('AcadLayers','Layers',AccessMode.ReadOnly)
    Layouts: AcadLayouts = proxy_property('AcadLayouts','Layouts',AccessMode.ReadOnly)
    Limits: vDoubleArray = proxy_property('vDoubleArray','Limits',AccessMode.ReadWrite)
    Linetypes: AcadLineTypes = proxy_property('AcadLineTypes','Linetypes',AccessMode.ReadOnly)
    Materials: AcadMaterials = proxy_property('AcadMaterials','Materials',AccessMode.ReadOnly)
    ModelSpace: AcadModelSpace = proxy_property('AcadModelSpace','ModelSpace',AccessMode.ReadOnly)
    PaperSpace: AcadPaperSpace = proxy_property('AcadPaperSpace','PaperSpace',AccessMode.ReadOnly)
    PlotConfigurations: AcadPlotConfigurations = proxy_property('AcadPlotConfigurations','PlotConfigurations',AccessMode.ReadOnly)
    Preferences: AcadDatabasePreferences = proxy_property('AcadDatabasePreferences','Preferences',AccessMode.ReadOnly)
    RegisteredApplications: AcadRegisteredApplications = proxy_property('AcadRegisteredApplications','RegisteredApplications',AccessMode.ReadOnly)
    SectionManager: AcadSectionManager = proxy_property('AcadSectionManager','SectionManager',AccessMode.ReadOnly)
    SummaryInfo: AcadSummaryInfo = proxy_property('AcadSummaryInfo','SummaryInfo',AccessMode.ReadOnly)
    TextStyles: AcadTextStyles = proxy_property('AcadTextStyles','TextStyles',AccessMode.ReadOnly)
    UserCoordinateSystems: AcadUCSs = proxy_property('AcadUCSs','UserCoordinateSystems',AccessMode.ReadOnly)
    Viewports: AcadViewports = proxy_property('AcadViewports','Viewports',AccessMode.ReadOnly)
    Views: AcadViews = proxy_property('AcadViews','Views',AccessMode.ReadOnly)

    def CopyObjects(self, Objects: vObjectArray, Owner: vObject = None) -> vObjectArray: #AcadIDPair
        IDPairs = vObjectEmpty
        return vObjectArray(self._obj.CopyObjects(Objects, Owner(), IDPairs()))

    def HandleToObject(self, Handle: str) -> AppObject: 
        return AppObject(self._obj.HandleToObject(Handle))

    def ObjectIdToObject(self, ID: int) -> AppObject: 
        return AppObject(self._obj.ObjectIDToObject(ID))
 
class IAcadDatabase(AcadDatabase):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)



from .AcadAcCmColor import *
from .AcadApplication import *
from .AcadBlock import *
from .AcadBlocks import *
from .AcadDatabase import *
from .AcadDatabasePreferences import *
from .AcadDictionaries import *
from .AcadDictionary import *
from .AcadDimStyle import *
from .AcadDimStyles import *
from .AcadDocument import *
from .AcadDocuments import *
from .AcadDynamicBlockReferenceProperty import *
from .AcadGroup import *
from .AcadGroups import *
from .AcadHyperlink import *
from .AcadHyperlinks import *
from .AcadIDPair import *
from .AcadLayer import *
from .AcadLayers import *
from .AcadLayerStateManager import *
from .AcadLayout import *
from .AcadLayouts import *
from .AcadLineType import *
from .AcadLineTypes import *
from .AcadMaterial import *
from .AcadMaterials import *
from .AcadMenuBar import *
from .AcadMenuGroup import *
from .AcadMenuGroups import *
from .AcadMLeaderLeader import *
from .AcadMLeaderStyle import *
from .AcadModelSpace import *
from .AcadPaperSpace import *
from .AcadPlot import *
from .AcadPlotConfiguration import *
from .AcadPlotConfigurations import *
from .AcadPopupMenu import *
from .AcadPopupMenuItem import *
from .AcadPopupMenus import *
from .AcadPreferences import *
from .AcadPreferencesDisplay import *
from .AcadPreferencesDrafting import *
from .AcadPreferencesFiles import *
from .AcadPreferencesOpenSave import *
from .AcadPreferencesOutput import *
from .AcadPreferencesProfiles import *
from .AcadPreferencesSelection import *
from .AcadPreferencesSystem import *
from .AcadPreferencesUser import *
from .AcadRegisteredApplication import *
from .AcadRegisteredApplications import *
from .AcadSectionManager import *
from .AcadSectionSettings import *
from .AcadSectionTypeSettings import *
from .AcadSecurityParams import *
from .AcadSelectionSet import *
from .AcadSelectionSets import *
from .AcadSortentsTable import *
from .AcadState import *
from .AcadSubDMeshEdge import *
from .AcadSubDMeshFace import *
from .AcadSubDMeshVertex import *
from .AcadSubEntity import *
from .AcadSubEntSolidEdge import *
from .AcadSubEntSolidFace import *
from .AcadSubEntSolidNode import *
from .AcadSubEntSolidVertex import *
from .AcadSummaryInfo import *
from .AcadTableStyle import *
from .AcadTextStyle import *
from .AcadTextStyles import *
from .AcadToolbar import *
from .AcadToolbarItem import *
from .AcadToolbars import *
from .AcadUCS import *
from .AcadUCSs import *
from .AcadUtility import *
from .AcadView import *
from .AcadViewport import *
from .AcadViewports import *
from .AcadViews import *
from .AcadXRecord import *

__all__ = ['AcadAcCmColor', 'AcadApplication', 'AcadBlock', 'AcadBlocks', 'AcadDatabase', 'IAcadDatabase', 'AcadDatabasePreferences', 'AcadDictionaries', 'AcadDictionary', 'AcadDimStyle', 'AcadDimStyles', 'AcadDocument', 'AcadDocuments', 'AcadDynamicBlockReferenceProperty', 'AcadGroup', 'AcadGroups', 'AcadHyperlink', 'AcadHyperlinks', 'AcadIDPair', 'AcadLayer', 'AcadLayers', 'AcadLayerStateManager', 'AcadLayout', 'AcadLayouts', 'AcadLineType', 'AcadLineTypes', 'AcadMaterial', 'AcadMaterials', 'AcadMenuBar', 'AcadMenuGroup', 'AcadMenuGroups', 'AcadMLeaderLeader', 'AcadMLeaderStyle', 'AcadModelSpace', 'AcadPaperSpace', 'AcadPlot', 'AcadPlotConfiguration', 'AcadPlotConfigurations', 'AcadPopupMenu', 'AcadPopupMenuItem', 'AcadPopupMenus', 'AcadPreferences', 'AcadPreferencesDisplay', 'AcadPreferencesDrafting', 'AcadPreferencesFiles', 'AcadPreferencesOpenSave', 'AcadPreferencesOutput', 'AcadPreferencesProfiles', 'AcadPreferencesSelection', 'AcadPreferencesSystem', 'AcadPreferencesUser', 'AcadRegisteredApplication', 'AcadRegisteredApplications', 'AcadSectionManager', 'AcadSectionSettings', 'AcadSectionTypeSettings', 'AcadSecurityParams', 'AcadSelectionSet', 'AcadSelectionSets', 'AcadSortentsTable', 'AcadState', 'AcadSubDMeshEdge', 'AcadSubDMeshFace', 'AcadSubDMeshVertex', 'AcadSubEntity', 'AcadSubEntSolidEdge', 'AcadSubEntSolidFace', 'AcadSubEntSolidNode', 'AcadSubEntSolidVertex', 'AcadSummaryInfo', 'AcadTableStyle', 'AcadTextStyle', 'AcadTextStyles', 'AcadToolbar', 'AcadToolbarItem', 'AcadToolbars', 'AcadUCS', 'AcadUCSs', 'AcadUtility', 'AcadView', 'AcadViewport', 'AcadViewports', 'AcadViews', 'AcadXRecord']

# Names in __all__ with no definition:
#   AcadAcCmColor
#   AcadApplication
#   AcadBlock
#   AcadBlocks
#   AcadDatabase
#   AcadDatabasePreferences
#   AcadDictionaries
#   AcadDictionary
#   AcadDimStyle
#   AcadDimStyles
#   AcadDocument
#   AcadDocuments
#   AcadDynamicBlockReferenceProperty
#   AcadGroup
#   AcadGroups
#   AcadHyperlink
#   AcadHyperlinks
#   AcadIDPair
#   AcadLayer
#   AcadLayerStateManager
#   AcadLayers
#   AcadLayout
#   AcadLayouts
#   AcadLineType
#   AcadLineTypes
#   AcadMLeaderLeader
#   AcadMLeaderStyle
#   AcadMaterial
#   AcadMaterials
#   AcadMenuBar
#   AcadMenuGroup
#   AcadMenuGroups
#   AcadModelSpace
#   AcadPaperSpace
#   AcadPlot
#   AcadPlotConfiguration
#   AcadPlotConfigurations
#   AcadPopupMenu
#   AcadPopupMenuItem
#   AcadPopupMenus
#   AcadPreferences
#   AcadPreferencesDisplay
#   AcadPreferencesDrafting
#   AcadPreferencesFiles
#   AcadPreferencesOpenSave
#   AcadPreferencesOutput
#   AcadPreferencesProfiles
#   AcadPreferencesSelection
#   AcadPreferencesSystem
#   AcadPreferencesUser
#   AcadRegisteredApplication
#   AcadRegisteredApplications
#   AcadSectionManager
#   AcadSectionSettings
#   AcadSectionTypeSettings
#   AcadSecurityParams
#   AcadSelectionSet
#   AcadSelectionSets
#   AcadSortentsTable
#   AcadState
#   AcadSubDMeshEdge
#   AcadSubDMeshFace
#   AcadSubDMeshVertex
#   AcadSubEntSolidEdge
#   AcadSubEntSolidFace
#   AcadSubEntSolidNode
#   AcadSubEntSolidVertex
#   AcadSubEntity
#   AcadSummaryInfo
#   AcadTableStyle
#   AcadTextStyle
#   AcadTextStyles
#   AcadToolbar
#   AcadToolbarItem
#   AcadToolbars
#   AcadUCS
#   AcadUCSs
#   AcadUtility
#   AcadView
#   AcadViewport
#   AcadViewports
#   AcadViews
#   AcadXRecord
#   IAcadDatabase



from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadBlock import IAcadBlock as IAcadBlock

class AcadModelSpace(IAcadBlock):
    def __init__(self, obj) -> None: ...
    Comments: str
    Layout: AcadLayout
    Name: str
    Origin: PyGePoint3d
    Units: AcInsertUnits



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadRasterImage import AcadRasterImage



class AcadGeomapImage(AcadRasterImage):
    def __init__(self, obj) -> None: super().__init__(obj)

    GeoImageBrightness: int = proxy_property(int,'GeoImageBrightness',AccessMode.ReadWrite)
    GeoImageContrast: int = proxy_property(int,'GeoImageContrast',AccessMode.ReadWrite)
    GeoImageFade: int = proxy_property(int,'GeoImageFade',AccessMode.ReadWrite)
    GeoImageHeight: float = proxy_property(float,'GeoImageHeight',AccessMode.ReadOnly)
    GeoImageWidth: float = proxy_property(float,'GeoImageWidth',AccessMode.ReadOnly)

    def Copy(self) -> AcadGeomapImage:
        return AcadGeomapImage(self._obj.Copy())



from ..Base import *
from ..Proxy import *

class AcadSummaryInfo(AppObject):
    def __init__(self, obj) -> None: ...
    Author: str
    Comments: str
    HyperlinkBase: str
    Keywords: str
    LastSavedBy: str
    RevisionNumber: str
    Subject: str
    Title: str
    def AddCustomInfo(self, key: str, Value: str) -> None: ...
    def GetCustomByIndex(self, Index: int) -> str: ...
    def GetCustomByKey(self, pKey: str) -> vStringArray: ...
    def NumCustomInfo(self) -> int: ...
    def RemoveCustomByIndex(self, Index: int) -> None: ...
    def RemoveCustomByKey(self, key: str) -> None: ...
    def SetCustomByIndex(self, Index: int, key: str, Value: str) -> None: ...
    def SetCustomByKey(self, key: str, Value: str) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *



class AcadSummaryInfo(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Author: str = proxy_property(str,'Author',AccessMode.ReadWrite)
    Comments: str = proxy_property(str,'Comments',AccessMode.ReadWrite)
    HyperlinkBase: str = proxy_property(str,'HyperlinkBase',AccessMode.ReadWrite)
    Keywords: str = proxy_property(str,'Keywords',AccessMode.ReadWrite)
    LastSavedBy: str = proxy_property(str,'LastSavedBy',AccessMode.ReadWrite)
    RevisionNumber: str = proxy_property(str,'RevisionNumber',AccessMode.ReadWrite)
    Subject: str = proxy_property(str,'Subject',AccessMode.ReadWrite)
    Title: str = proxy_property(str,'Title',AccessMode.ReadWrite)

    def AddCustomInfo(self, key: str, Value: str) -> None:
        self._obj.AddCustomInfo(key, Value)

    def GetCustomByIndex(self, Index: int) -> str:
        pKey, pValue = self._obj.GetCustomByIndex(Index)
        return pKey, pValue

    def GetCustomByKey(self, pKey: str) -> vStringArray:
        pValue = self._obj.GetCustomByKey(pKey)
        return vStringArray(pValue)

    def NumCustomInfo(self) -> int:
        return self._obj.NumCustomInfo()

    def RemoveCustomByIndex(self, Index: int) -> None:
        self._obj.RemoveCustomByIndex(Index)

    def RemoveCustomByKey(self, key: str) -> None:
        self._obj.RemoveCustomByKey(key)

    def SetCustomByIndex(self, Index: int, key: str, Value: str) -> None:
        self._obj.SetCustomByIndex(Index, key, Value)

    def SetCustomByKey(self, key: str, Value: str) -> None:
        self._obj.SetCustomByKey(key, Value)


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadRasterImage import AcadRasterImage



class AcadWipeout(AcadRasterImage):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Copy(self) -> AcadWipeout:
        return AcadWipeout(self._obj.Copy())


from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadViewport(AcadObject):
    def __init__(self, obj) -> None: ...
    ArcSmoothness: int
    Center: PyGePoint2d
    Direction: PyGeVector3d
    GridOn: bool
    Height: float
    LowerLeftCorner: vDoubleArray
    Name: str
    OrthoOn: bool
    SnapBasePoint: vDoubleArray
    SnapOn: bool
    SnapRotationAngle: float
    Target: PyGePoint3d
    UCSIconAtOrigin: bool
    UCSIconOn: bool
    UpperRightCorner: vDoubleArray
    Width: float
    def GetGridSpacing(self) -> vDoubleArray: ...
    def GetSnapSpacing(self) -> vDoubleArray: ...
    def SetGridSpacing(self, XSpacing: float, YSpacing: float) -> None: ...
    def SetSnapSpacing(self, XSpacing: float, YSpacing: float) -> None: ...
    def SetView(self, View: AcadView) -> None: ...
    def Split(self, NumWins: AcViewportSplitType) -> None: ...



from ..Base import *
from ..Proxy import *
from .AcadToolbar import *

class AcadToolbars(AppObjectCollection):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Count: int
    LargeButtons: bool
    Parent: AppObject
    def Add(self, Name: str) -> AcadToolbar: ...
    def Item(self, Index: int | str) -> AcadToolbar: ...
    def __iter__(self): ...



from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadSubEntity(AppObject):
    def __init__(self, obj) -> None: ...
    Color: AcColor
    Hyperlinks: AcadHyperlinks
    Layer: str
    Linetype: str
    LinetypeScale: float
    Lineweight: AcLineWeight
    ObjectName: str
    PlotStyleName: str



from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadRegisteredApplication(AcadObject):
    def __init__(self, obj) -> None: ...
    Name: str



from ..VarType import *
from .Matrix import *
from .Vector import *

class PyGePoint3d(vDoubleArray):
    def __init__(self, *args) -> None: ...
    @property
    def x(self): ...
    @x.setter
    def x(self, value) -> None: ...
    @property
    def y(self): ...
    @y.setter
    def y(self, value) -> None: ...
    @property
    def z(self): ...
    @z.setter
    def z(self, value) -> None: ...
    def __add__(self, vector: PyGeVector3d) -> PyGePoint3d: ...
    def __sub__(self, other: PyGePoint3d | PyGeVector3d) -> PyGeVector3d | PyGePoint3d: ...
    def __mul__(self, scalar: int | float) -> PyGePoint3d: ...
    def __rmul__(self, scalar: int | float) -> PyGePoint3d: ...
    def __truediv__(self, scalar: int | float) -> PyGePoint3d: ...
    def __iadd__(self, vector: PyGeVector3d) -> PyGePoint3d: ...
    def __isub__(self, vector: PyGeVector3d) -> PyGePoint3d: ...
    def __imul__(self, scalar: int | float) -> PyGePoint3d: ...
    def __itruediv__(self, scalar: int | float) -> PyGePoint3d: ...
    def asVector(self) -> PyGeVector3d: ...
    def distanceTo(self, other: PyGePoint3d) -> float: ...
    def isEqualTo(self, other_point: PyGePoint3d, tolerance: float = 1e-09) -> bool: ...
    def transformBy(self, matrix: PyGeMatrix3d) -> None: ...

class PyGePoint2d(vDoubleArray):
    def __init__(self, *args) -> None: ...
    @property
    def x(self): ...
    @x.setter
    def x(self, value) -> None: ...
    @property
    def y(self): ...
    @y.setter
    def y(self, value) -> None: ...
    def __add__(self, vector: PyGeVector2d) -> PyGePoint2d: ...
    def __sub__(self, other: PyGePoint2d | PyGeVector2d) -> PyGeVector2d | PyGePoint2d: ...
    def __mul__(self, scalar: int | float) -> PyGePoint2d: ...
    def __rmul__(self, scalar: int | float) -> PyGePoint2d: ...
    def __truediv__(self, scalar: int | float) -> PyGePoint2d: ...
    def __iadd__(self, vector: PyGeVector2d) -> PyGePoint2d: ...
    def __isub__(self, vector: PyGeVector2d) -> PyGePoint2d: ...
    def __imul__(self, scalar: int | float) -> PyGePoint2d: ...
    def __itruediv__(self, scalar: int | float) -> PyGePoint2d: ...
    def asVector(self) -> PyGeVector2d: ...
    def distanceTo(self, other: PyGePoint2d) -> float: ...
    def isEqualTo(self, other: PyGePoint2d, tolerance: float = 1e-09) -> bool: ...
    def transformBy(self, matrix: PyGeMatrix2d) -> PyGePoint2d: ...

class PyGePoint3dArray(vDoubleArray):
    def __init__(self, *args) -> None: ...
    def __iter__(self): ...
    def __getitem__(self, index): ...
    def __setitem__(self, index, value) -> None: ...
    def __len__(self) -> int: ...
    def __add__(self, other): ...

class PyGePoint2dArray(vDoubleArray):
    def __init__(self, *args) -> None: ...
    def __iter__(self): ...
    def __getitem__(self, index): ...
    def __setitem__(self, index, value) -> None: ...
    def __len__(self) -> int: ...
    def __add__(self, other): ...



from .Acad3DFace import *
from .Acad3DPolyline import *
from .Acad3DSolid import *
from .AcadArc import *
from .AcadAttribute import *
from .AcadAttributeReference import *
from .AcadBlockReference import *
from .AcadCircle import *
from .AcadComparedReference import *
from .AcadDgnUnderlay import *
from .AcadDim3PointAngular import *
from .AcadDimAligned import *
from .AcadDimAngular import *
from .AcadDimArcLength import *
from .AcadDimDiametric import *
from .AcadDimension import *
from .AcadDimOrdinate import *
from .AcadDimRadial import *
from .AcadDimRadialLarge import *
from .AcadDimRotated import *
from .AcadDwfUnderlay import *
from .AcadEllipse import *
from .AcadExternalReference import *
from .AcadExtrudedSurface import *
from .AcadGeomapImage import *
from .AcadGeoPositionMarker import *
from .AcadHatch import *
from .AcadHelix import *
from .AcadLeader import *
from .AcadLine import *
from .AcadLoftedSurface import *
from .AcadLWPolyline import *
from .AcadMInsertBlock import *
from .AcadMLeader import *
from .AcadMLine import *
from .AcadMtext import *
from .AcadNurbSurface import *
from .AcadOle import *
from .AcadPdfUnderlay import *
from .AcadPlaneSurface import *
from .AcadPoint import *
from .AcadPointCloud import *
from .AcadPointCloudEx import *
from .AcadPolyfaceMesh import *
from .AcadPolygonMesh import *
from .AcadPolyline import *
from .AcadPViewport import *
from .AcadRasterImage import *
from .AcadRay import *
from .AcadRegion import *
from .AcadSection import *
from .AcadShape import *
from .AcadSolid import *
from .AcadSpline import *
from .AcadSubDMesh import *
from .AcadSurface import *
from .AcadSweptSurface import *
from .AcadTable import *
from .AcadText import *
from .AcadTolerance import *
from .AcadTrace import *
from .AcadUnderlay import *
from .AcadWipeout import *
from .AcadXline import *

__all__ = [
    'Acad3DFace',
    'Acad3DPolyline',
    'Acad3DSolid',
    'AcadArc',
    'AcadAttribute',
    'AcadAttributeReference',
    'AcadBlockReference',
    'AcadCircle',
    'AcadComparedReference',
    'AcadDgnUnderlay',
    'AcadDim3PointAngular',
    'AcadDimAligned',
    'AcadDimAngular',
    'AcadDimArcLength',
    'AcadDimDiametric',
    'AcadDimension',
    'AcadDimOrdinate',
    'AcadDimRadial',
    'AcadDimRadialLarge',
    'AcadDimRotated',
    'AcadDwfUnderlay',
    'AcadEllipse',
    'AcadExternalReference',
    'AcadExtrudedSurface',
    'AcadGeomapImage',
    'AcadGeoPositionMarker',
    'AcadHatch',
    'AcadHelix',
    'AcadLeader',
    'AcadLine',
    'AcadLoftedSurface',
    'AcadLWPolyline',
    'AcadMInsertBlock',
    'AcadMLeader',
    'AcadMLine',
    'AcadMtext',
    'AcadNurbSurface',
    'AcadOle',
    'AcadPdfUnderlay',
    'AcadPlaneSurface',
    'AcadPoint',
    'AcadPointCloud',
    'AcadPointCloudEx',
    'AcadPolyfaceMesh',
    'AcadPolygonMesh',
    'AcadPolyline',
    'AcadPViewport',
    'AcadRasterImage',
    'AcadRay',
    'AcadRegion',
    'AcadSection',
    'AcadShape',
    'AcadSolid',
    'AcadSpline',
    'AcadSubDMesh',
    'AcadSurface',
    'AcadSweptSurface',
    'AcadTable',
    'AcadText',
    'AcadTolerance',
    'AcadTrace',
    'AcadUnderlay',
    'AcadWipeout',
    'AcadXline'
]


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadPointCloud(AcadEntity):
    def __init__(self, obj) -> None: ...
    Height: float
    InsertionPoint: PyGePoint3d
    IntensityColorScheme: AcPointCloudIntensityStyle
    Locked: bool
    Name: str
    Path: str
    Rotation: float
    Scale: float
    ShowClipped: bool
    ShowIntensity: bool
    Stylization: AcPointCloudStylizationType
    Unit: str
    UnitFactor: str
    UseEntityColor: AcPointCloudColorType
    Width: float
    def Copy(self) -> AcadPointCloud: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadPreferencesDrafting(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    AlignmentPointAcquisition: AcAlignmentPointAcquisition = proxy_property('AcAlignmentPointAcquisition','AlignmentPointAcquisition',AccessMode.ReadWrite)
    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    AutoSnapAperture: bool = proxy_property(bool,'AutoSnapAperture',AccessMode.ReadWrite)
    AutoSnapApertureSize: int = proxy_property(int,'AutoSnapApertureSize',AccessMode.ReadWrite)
    AutoSnapMagnet: bool = proxy_property(bool,'AutoSnapMagnet',AccessMode.ReadWrite)
    AutoSnapMarker: bool = proxy_property(bool,'AutoSnapMarker',AccessMode.ReadWrite)
    AutoSnapMarkerColor: AcColor = proxy_property('AcColor','AutoSnapMarkerColor',AccessMode.ReadWrite)
    AutoSnapMarkerSize: int = proxy_property(int,'AutoSnapMarkerSize',AccessMode.ReadWrite)
    AutoSnapToolTip: bool = proxy_property(bool,'AutoSnapToolTip',AccessMode.ReadWrite)
    AutoTrackTooltip: bool = proxy_property(bool,'AutoTrackTooltip',AccessMode.ReadWrite)
    FullScreenTrackingVector: bool = proxy_property(bool,'FullScreenTrackingVector',AccessMode.ReadWrite)
    PolarTrackingVector: bool = proxy_property(bool,'PolarTrackingVector',AccessMode.ReadWrite)


from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadPreferencesUser(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    ADCInsertUnitsDefaultSource: AcInsertUnits = proxy_property('AcInsertUnits','ADCInsertUnitsDefaultSource',AccessMode.ReadWrite)
    ADCInsertUnitsDefaultTarget: AcInsertUnits = proxy_property('AcInsertUnits','ADCInsertUnitsDefaultTarget',AccessMode.ReadWrite)
    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    HyperlinkDisplayCursor: bool = proxy_property(bool,'HyperlinkDisplayCursor',AccessMode.ReadWrite)
    KeyboardAccelerator: AcKeyboardAccelerator = proxy_property('AcKeyboardAccelerator','KeyboardAccelerator',AccessMode.ReadWrite)
    KeyboardPriority: AcKeyboardPriority = proxy_property('AcKeyboardPriority','KeyboardPriority',AccessMode.ReadWrite)
    SCMCommandMode: AcDrawingAreaSCMCommand = proxy_property('AcDrawingAreaSCMCommand','SCMCommandMode',AccessMode.ReadWrite)
    SCMDefaultMode: AcDrawingAreaSCMDefault = proxy_property('AcDrawingAreaSCMDefault','SCMDefaultMode',AccessMode.ReadWrite)
    SCMEditMode: AcDrawingAreaSCMEdit = proxy_property('AcDrawingAreaSCMEdit','SCMEditMode',AccessMode.ReadWrite)
    SCMTimeMode: bool = proxy_property(bool,'SCMTimeMode',AccessMode.ReadWrite)
    SCMTimeValue: int = proxy_property(int,'SCMTimeValue',AccessMode.ReadWrite)
    ShortCutMenuDisplay: bool = proxy_property(bool,'ShortCutMenuDisplay',AccessMode.ReadWrite)



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadUnderlay import AcadUnderlay



class AcadDwfUnderlay(AcadUnderlay):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Copy(self) -> AcadDwfUnderlay:
        return AcadDwfUnderlay(self._obj.Copy())



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadLineTypes(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Add(self, Name: str) -> AcadLineType: 
        return AcadLineType(self._obj.Add(Name))

    def Item(self, Index: int | str) -> AcadLineType:
        return AcadLineType(self._obj.Item(Index))
        
    def Load(self, LineTypeName: str, FileName: str) -> None:
        self._obj.Load(LineTypeName, FileName)

    def __iter__(self):
        for item in self._obj:
            yield AcadLineType(item)


from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadPreferencesDisplay(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    AutoTrackingVecColor: int = proxy_property('OLE_COLOR','AutoTrackingVecColor',AccessMode.ReadWrite)
    CursorSize: int = proxy_property(int,'CursorSize',AccessMode.ReadWrite)
    DisplayLayoutTabs: bool = proxy_property(bool,'DisplayLayoutTabs',AccessMode.ReadWrite)
    DisplayScreenMenu: bool = proxy_property(bool,'DisplayScreenMenu',AccessMode.ReadWrite)
    DisplayScrollBars: bool = proxy_property(bool,'DisplayScrollBars',AccessMode.ReadWrite)
    DockedVisibleLines: int = proxy_property(int,'DockedVisibleLines',AccessMode.ReadWrite)
    GraphicsWinLayoutBackgrndColor: int = proxy_property('OLE_COLOR','GraphicsWinLayoutBackgrndColor',AccessMode.ReadWrite)
    GraphicsWinModelBackgrndColor: int = proxy_property('OLE_COLOR','GraphicsWinModelBackgrndColor',AccessMode.ReadWrite)
    HistoryLines: int = proxy_property(int,'HistoryLines',AccessMode.ReadWrite)
    ImageFrameHighlight: bool = proxy_property(bool,'ImageFrameHighlight',AccessMode.ReadWrite)
    LayoutCreateViewport: bool = proxy_property(bool,'LayoutCreateViewport',AccessMode.ReadWrite)
    LayoutCrosshairColor: int = proxy_property('OLE_COLOR','LayoutCrosshairColor',AccessMode.ReadWrite)
    LayoutDisplayMargins: bool = proxy_property(bool,'LayoutDisplayMargins',AccessMode.ReadWrite)
    LayoutDisplayPaper: bool = proxy_property(bool,'LayoutDisplayPaper',AccessMode.ReadWrite)
    LayoutDisplayPaperShadow: bool = proxy_property(bool,'LayoutDisplayPaperShadow',AccessMode.ReadWrite)
    MaxAutoCADWindow: bool = proxy_property(bool,'MaxAutoCADWindow',AccessMode.ReadWrite)
    ModelCrosshairColor: int = proxy_property('OLE_COLOR','ModelCrosshairColor',AccessMode.ReadWrite)
    ShowRasterImage: bool = proxy_property(bool,'ShowRasterImage',AccessMode.ReadWrite)
    TextFont: str = proxy_property(str,'TextFont',AccessMode.ReadWrite)
    TextFontSize: int = proxy_property(int,'TextFontSize',AccessMode.ReadWrite)
    TextFontStyle: AcTextFontStyle = proxy_property('AcTextFontStyle','TextFontStyle',AccessMode.ReadWrite)
    TextWinBackgrndColor: int = proxy_property('OLE_COLOR','TextWinBackgrndColor',AccessMode.ReadWrite)
    TextWinTextColor: int = proxy_property('OLE_COLOR','TextWinBackgrndColor',AccessMode.ReadWrite)
    TrueColorImages: bool = proxy_property(bool,'TrueColorImages',AccessMode.ReadWrite)
    XRefFadeIntensity: int = proxy_property(int,'XRefFadeIntensity',AccessMode.ReadWrite)


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadBlockReference import AcadBlockReference



class AcadExternalReference(AcadBlockReference):
    def __init__(self, obj) -> None: super().__init__(obj)

    LayerPropertyOverrides: bool = proxy_property(bool,'LayerPropertyOverrides',AccessMode.ReadOnly)
    Path: str = proxy_property(str,'Path',AccessMode.ReadWrite)

    def Copy(self) -> AcadExternalReference:
        return AcadExternalReference(self._obj.Copy())


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadPlot(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    BatchPlotProgress: bool = proxy_property(bool,'BatchPlotProgress',AccessMode.ReadWrite)
    NumberOfCopies: int = proxy_property(int,'NumberOfCopies',AccessMode.ReadWrite)
    QuietErrorMode: bool = proxy_property(bool,'QuietErrorMode',AccessMode.ReadWrite)


    def DisplayPlotPreview(self, Preview: AcPreviewMode) -> None: 
        self._obj.DisplayPlotPreview(Preview)

    def PlotToDevice(self, plotConfig: str = '') -> bool: 
        return self._obj.PlotToDevice(plotConfig)

    def PlotToFile(self, plotFile: str, plotConfig: str = '') -> bool: 
        return self._obj.PlotToFile(plotFile, plotConfig)

    def SetLayoutsToPlot(self, layoutList: vStringArray) -> None: 
        self._obj.SetLayoutsToPlot(layoutList)
    
    def StartBatchMode(self, entryCount: int) -> None: 
        self._obj.StartBatchMode(entryCount)



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadLayout import *



class AcadLayouts(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Add(self, InsertionPoint: PyGePoint3d, Name: str) -> AcadLayout:
        return AcadLayout(self._obj.Add(InsertionPoint(), Name))

    def Item(self, Index: int) -> AcadLayout:
        return AcadLayout(self._obj.Item(Index))

    def __iter__(self):
        for item in self._obj:
            obj = AcadLayout(item)
            yield obj



from ..Base import *
from ..Proxy import *
from .AcadDimension import AcadDimension as AcadDimension

class AcadDimDiametric(AcadDimension):
    def __init__(self, obj) -> None: ...
    AltRoundDistance: float
    AltSuppressLeadingZeros: bool
    AltSuppressTrailingZeros: bool
    AltSuppressZeroFeet: bool
    AltSuppressZeroInches: bool
    AltTextPrefix: str
    AltTextSuffix: str
    AltTolerancePrecision: AcDimPrecision
    AltToleranceSuppressLeadingZeros: bool
    AltToleranceSuppressTrailingZeros: bool
    AltToleranceSuppressZeroFeet: bool
    AltToleranceSuppressZeroInches: bool
    AltUnits: bool
    AltUnitsFormat: AcDimUnits
    AltUnitsPrecision: AcDimPrecision
    AltUnitsScale: float
    Arrowhead1Block: str
    Arrowhead1Type: AcDimArrowheadType
    Arrowhead2Block: str
    Arrowhead2Type: AcDimArrowheadType
    ArrowheadSize: int
    CenterMarkSize: float
    CenterType: AcDimCenterType
    DimConstrDesc: str
    DimConstrExpression: str
    DimConstrForm: bool
    DimConstrName: str
    DimConstrReference: bool
    DimConstrValue: str
    DimensionLineColor: AcColor
    DimensionLinetype: str
    DimensionLineWeight: AcLineWeight
    DimLine1Suppress: bool
    DimLine2Suppress: bool
    Fit: AcDimFit
    ForceLineInside: bool
    FractionFormat: AcDimFractionType
    LeaderLength: float
    LinearScaleFactor: float
    Measurement: float
    PrimaryUnitsPrecision: AcDimPrecision
    RoundDistance: float
    SuppressZeroFeet: bool
    SuppressZeroInches: bool
    TextInside: bool
    TextInsideAlign: bool
    TextOutsideAlign: bool
    ToleranceSuppressZeroFeet: bool
    ToleranceSuppressZeroInches: bool
    UnitsFormat: AcDimLUnits
    def Copy(self) -> AcadDimDiametric: ...



from .Autocad.Objects import *
from .Autocad.Entities import *
from .Autocad.Proxy import *
from .Autocad.Base import *
from .Autocad.AcadEntity import *
from .Autocad.AcadObject import *
from .Types import *
from .Utility import *
__version__ = "0.0.1b3"
__all__ = [
    'AutoCAD',
    'BlockReference',
    'PyGePoint3d',
    'PyGePoint3dArray',
    'PyGePoint2d',
    'PyGePoint2dArray',
    'PyGeVector3d',
    'PyGeVector2d',
    'PyGeMatrix3d',
    'PyGeMatrix2d',
    'AcadAcCmColor',
    'AcadApplication',
    'AcadBlock',
    'AcadBlocks',
    'AcadDatabase',
    'IAcadDatabase',
    'AcadDatabasePreferences',
    'AcadDictionaries',
    'AcadDictionary',
    'AcadDimStyle',
    'AcadDimStyles',
    'AcadDocument',
    'AcadDocuments',
    'AcadDynamicBlockReferenceProperty',
    'AcadEntity',
    'AcadGroup',
    'AcadGroups',
    'AcadHyperlink',
    'AcadHyperlinks',
    'AcadIDPair',
    'AcadLayer',
    'AcadLayers',
    'AcadLayerStateManager',
    'AcadLayout',
    'AcadLayouts',
    'AcadLineType',
    'AcadLineTypes',
    'AcadMaterial',
    'AcadMaterials',
    'AcadMenuBar',
    'AcadMenuGroup',
    'AcadMenuGroups',
    'AcadMLeaderLeader',
    'AcadMLeaderStyle',
    'AcadModelSpace',
    'AcadObject',
    'AppObject',
    'AcadPaperSpace',
    'AcadPlot',
    'AcadPlotConfiguration',
    'AcadPlotConfigurations',
    'AcadPopupMenu',
    'AcadPopupMenuItem',
    'AcadPopupMenus',
    'AcadPreferences',
    'AcadPreferencesDisplay',
    'AcadPreferencesDrafting',
    'AcadPreferencesFiles',
    'AcadPreferencesOpenSave',
    'AcadPreferencesOutput',
    'AcadPreferencesProfiles',
    'AcadPreferencesSelection',
    'AcadPreferencesSystem',
    'AcadPreferencesUser',
    'AcadRegisteredApplication',
    'AcadRegisteredApplications',
    'AcadSectionManager',
    'AcadSectionSettings',
    'AcadSectionTypeSettings',
    'AcadSecurityParams',
    'AcadSelectionSet',
    'AcadSelectionSets',
    'AcadSortentsTable',
    'AcadState',
    'AcadSubDMeshEdge',
    'AcadSubDMeshFace',
    'AcadSubDMeshVertex',
    'AcadSubEntity',
    'AcadSubEntSolidEdge',
    'AcadSubEntSolidFace',
    'AcadSubEntSolidNode',
    'AcadSubEntSolidVertex',
    'AcadSummaryInfo',
    'AcadTableStyle',
    'AcadTextStyle',
    'AcadTextStyles',
    'AcadToolbar',
    'AcadToolbarItem',
    'AcadToolbars',
    'AcadUCS',
    'AcadUCSs',
    'AcadUtility',
    'AcadView',
    'AcadViewport',
    'AcadViewports',
    'AcadViews',
    'AcadXRecord',
    'Acad3DFace',
    'Acad3DPolyline',
    'Acad3DSolid',
    'AcadArc',
    'AcadAttribute',
    'AcadAttributeReference',
    'AcadBlockReference',
    'AcadCircle',
    'AcadComparedReference',
    'AcadDgnUnderlay',
    'AcadDim3PointAngular',
    'AcadDimAligned',
    'AcadDimAngular',
    'AcadDimArcLength',
    'AcadDimDiametric',
    'AcadDimension',
    'AcadDimOrdinate',
    'AcadDimRadial',
    'AcadDimRadialLarge',
    'AcadDimRotated',
    'AcadDwfUnderlay',
    'AcadEllipse',
    'AcadExternalReference',
    'AcadExtrudedSurface',
    'AcadGeomapImage',
    'AcadGeoPositionMarker',
    'AcadHatch',
    'AcadHelix',
    'AcadLeader',
    'AcadLine',
    'AcadLoftedSurface',
    'AcadLWPolyline',
    'AcadMInsertBlock',
    'AcadMLeader',
    'AcadMLine',
    'AcadMtext',
    'AcadNurbSurface',
    'AcadOle',
    'AcadPdfUnderlay',
    'AcadPlaneSurface',
    'AcadPoint',
    'AcadPointCloud',
    'AcadPointCloudEx',
    'AcadPolyfaceMesh',
    'AcadPolygonMesh',
    'AcadPolyline',
    'AcadPViewport',
    'AcadRasterImage',
    'AcadRay',
    'AcadRegion',
    'AcadSection',
    'AcadShape',
    'AcadSolid',
    'AcadSpline',
    'AcadSubDMesh',
    'AcadSurface',
    'AcadSweptSurface',
    'AcadTable',
    'AcadText',
    'AcadTolerance',
    'AcadTrace',
    'AcadUnderlay',
    'AcadWipeout',
    'AcadXline',
    'Ac3DPolylineType',
    'AcARXDemandLoad',
    'AcActiveSpace',
    'AcAlignment',
    'AcAlignmentPointAcquisition',
    'AcAngleUnits',
    'AcAttachmentPoint',
    'AcAttributeMode',
    'AcBlockConnectionType',
    'AcBlockScaling',
    'AcBoolean',
    'AcBooleanType',
    'AcCellAlignment',
    'AcCellContentLayout',
    'AcCellContentType',
    'AcCellEdgeMask',
    'AcCellMargin',
    'AcCellOption',
    'AcCellProperty',
    'AcCellState',
    'AcCellType',
    'AcColor',
    'AcColorMethod',
    'AcCoordinateSystem',
    'AcDataLinkUpdateDirection',
    'AcDataLinkUpdateOption',
    'AcDimArcLengthSymbol',
    'AcDimArrowheadType',
    'AcDimCenterType',
    'AcDimFit',
    'AcDimFractionType',
    'AcDimHorizontalJustification',
    'AcDimLUnits',
    'AcDimPrecision',
    'AcDimTextMovement',
    'AcDimToleranceJustify',
    'AcDimToleranceMethod',
    'AcDimUnits',
    'AcDimVerticalJustification',
    'AcDragDisplayMode',
    'AcDrawLeaderOrderType',
    'AcDrawMLeaderOrderType',
    'AcDrawingAreaSCMCommand',
    'AcDrawingAreaSCMDefault',
    'AcDrawingAreaSCMEdit',
    'AcDrawingAreaShortCutMenu',
    'AcDrawingDirection',
    'AcDynamicBlockReferencePropertyUnitsType',
    'AcEntityName',
    'AcExtendOption',
    'AcFormatOption',
    'AcGradientPatternType',
    'AcGridLineStyle',
    'AcGridLineType',
    'AcHatchObjectType',
    'AcHatchStyle',
    'AcHelixConstrainType',
    'AcHelixTwistType',
    'AcHorizontalAlignment',
    'AcISOPenWidth',
    'AcInsertUnits',
    'AcInsertUnitsAction',
    'AcKeyboardAccelerator',
    'AcKeyboardPriority',
    'AcLayerStateMask',
    'AcLeaderType',
    'AcLineSpacingStyle',
    'AcLineWeight',
    'AcLoadPalette',
    'AcLoftedSurfaceNormalType',
    'AcLoopType',
    'AcMLeaderContentType',
    'AcMLeaderType',
    'AcMLineJustification',
    'AcMeasurementUnits',
    'AcMenuFileType',
    'AcMenuGroupType',
    'AcMenuItemType',
    'AcMergeCellStyleOption',
    'AcMeshCreaseType',
    'AcOlePlotQuality',
    'AcOleQuality',
    'AcOleType',
    'AcOnOff',
    'AcParseOption',
    'AcPatternType',
    'AcPlotOrientation',
    'AcPlotPaperUnits',
    'AcPlotPolicy',
    'AcPlotPolicyForLegacyDwgs',
    'AcPlotPolicyForNewDwgs',
    'AcPlotRotation',
    'AcPlotScale',
    'AcPlotType',
    'AcPointCloudColorType',
    'AcPointCloudExStylizationType',
    'AcPointCloudIntensityStyle',
    'AcPointCloudStylizationType',
    'AcPolylineType',
    'AcPolymeshType',
    'AcPredefBlockType',
    'AcPreviewMode',
    'AcPrinterSpoolAlert',
    'AcProxyImage',
    'AcRegenType',
    'AcRotationAngle',
    'AcRowType',
    'AcSaveAsType',
    'AcSectionGeneration',
    'AcSectionState',
    'AcSectionState2',
    'AcSectionSubItem',
    'AcSectionType',
    'AcSegmentAngleType',
    'AcSelect',
    'AcSelectType',
    'AcShadePlot',
    'AcShadowDisplayType',
    'AcSplineFrameType',
    'AcSplineKnotParameterizationType',
    'AcSplineMethodType',
    'AcTableDirection',
    'AcTableFlowDirection',
    'AcTableStyleOverrides',
    'AcTextAlignmentType',
    'AcTextAngleType',
    'AcTextAttachmentDirection',
    'AcTextAttachmentType',
    'AcTextFontStyle',
    'AcTextGenerationFlag',
    'AcToolbarDockStatus',
    'AcToolbarItemType',
    'AcUnderlayLayerOverrideType',
    'AcUnits',
    'AcValueDataType',
    'AcValueUnitType',
    'AcVerticalAlignment',
    'AcVerticalTextAttachmentType',
    'AcViewportScale',
    'AcViewportSplitType',
    'AcWindowState',
    'AcWireframeType',
    'AcXRefDemandLoad',
    'AcZoomScaleType',
    'AcadSecurityParamsConstants',
    'AcadSecurityParamsType',
    'AdeskDxfCode',
    'AcEntityTransparency',
    'uArrowSymbol',
    'is_iterable',
    'flatten_generic',
    'VT',
    'Variant',
    'vObjectEmpty',
    'vObject',
    'vDouble',
    'vLong',
    'vInteger',
    'vShort',
    'vBool',
    'vString',
    'vArray',
    'vObjectArray',
    'vVariantArray',
    'vDoubleArray',
    'vLongArray',
    'vStringArray',
    'vIntegerArray',
    'vBoolArray',
    'vShortArray',
    'DxfGroupXDCode',
    'XDataManager',
    'AccessMode',
    'com_server_is_running',
    'get_clsid',
    'create_new_instance_explicitly',
    'clear_com_cache'
]



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *



class AcadXRecord(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    TranslateIDs: bool = proxy_property(bool,'TranslateIDs',AccessMode.ReadWrite)

    def GetXRecordData(self) -> tuple: 
        XRecordDataType, XRecordDataValue = self._obj.GetXRecordData()
        return XRecordDataType, XRecordDataValue

    def SetXRecordData(self, XRecordDataType: Variant, XRecordData: Variant) -> None: 
        self._obj.SetXRecordData(XRecordDataType(), XRecordData())



from __future__ import annotations
from ..Base import *
from ..AcadObject import *
from .AcadTextStyle import *


class AcadTextStyles(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Add(self, Name: str = None) -> AcadTextStyle:
        return AcadTextStyle(self._obj.Add(Name))

    def Item(self, Index: int) -> AcadTextStyle:
        return AcadTextStyle(self._obj.Item(Index))

    def __iter__(self):
        for item in self._obj:
            obj = AcadTextStyle(item)
            yield obj



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadSpline(AcadEntity):
    def __init__(self, obj) -> None: ...
    Area: float
    Closed: bool
    Closed2: bool
    ControlPoints: PyGePoint3dArray
    Degree: int
    Degree2: int
    EndTangent: PyGeVector3d
    FitPoints: PyGePoint3dArray
    FitTolerance: float
    IsPeriodic: bool
    IsPlanar: bool
    IsRational: bool
    KnotParameterization: AcSplineKnotParameterizationType
    Knots: PyGeVector3d
    NumberOfControlPoints: int
    NumberOfFitPoints: int
    SplineFrame: AcSplineFrameType
    SplineMethod: AcSplineMethodType
    StartTangent: PyGeVector3d
    Weights: PyGeVector3d
    def AddFitPoint(self, Index: int, FitPoint: PyGePoint3d) -> None: ...
    def Copy(self) -> AcadSpline: ...
    def DeleteFitPoint(self, Index: int) -> None: ...
    def ElevateOrder(self, Order: int) -> None: ...
    def GetControlPoint(self, Index: int) -> PyGePoint3d: ...
    def GetFitPoint(self, Index: int) -> PyGePoint3d: ...
    def GetWeight(self, Index: int) -> int: ...
    def Offset(self, Distance: float) -> AcadSpline: ...
    def PurgeFitData(self) -> None: ...
    def Reverse(self) -> None: ...
    def SetControlPoint(self, Index: int, Value: PyGePoint3d) -> None: ...
    def SetFitPoint(self, Index: int, Value: PyGePoint3d) -> None: ...
    def SetWeight(self, Index: int, Weight: float) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *

class AcadHyperlink(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    URL: str = proxy_property(str,'URL',AccessMode.ReadWrite)
    URLDescription: str = proxy_property(str,'URLDescription',AccessMode.ReadWrite)
    URLNamedLocation: str = proxy_property(str,'URLNamedLocation',AccessMode.ReadWrite)

    def Delete(self) -> None: 
        self._obj.Delete()


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadPlotConfiguration import *



class AcadPlotConfigurations(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Add(self, Name: str, ModelType: bool = None) -> AcadPlotConfiguration:
        if ModelType is not None:
            return self._obj.Add(Name, ModelType)
        else:
            return self._obj.Add(Name)

    def Item(self, Index: int | str) -> AcadPlotConfiguration:
        return AcadPlotConfiguration(self._obj.Item(Index))

    def __iter__(self):
        for item in self._obj:
            obj = AcadPlotConfiguration(item)
            yield obj


from .Points import *
from .Matrix import *
from .Vector import *

__all__ = ['PyGePoint3d', 'PyGePoint3dArray', 'PyGePoint2d', 'PyGePoint2dArray', 'PyGeVector3d', 'PyGeVector2d', 'PyGeMatrix3d', 'PyGeMatrix2d']

# Names in __all__ with no definition:
#   PyGeMatrix2d
#   PyGeMatrix3d
#   PyGePoint2d
#   PyGePoint2dArray
#   PyGePoint3d
#   PyGePoint3dArray
#   PyGeVector2d
#   PyGeVector3d



from ..Base import *
from ..Proxy import *
from .AcadBlockReference import AcadBlockReference as AcadBlockReference

class AcadExternalReference(AcadBlockReference):
    def __init__(self, obj) -> None: ...
    LayerPropertyOverrides: bool
    Path: str
    def Copy(self) -> AcadExternalReference: ...



from ..Base import *
from ..AcadObject import *
from .AcadMenuGroup import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadMenuGroups(AppObjectCollection):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Count: int
    Parent: AppObject
    def Item(self, Index: int | str) -> AcadMenuGroup: ...
    def Load(self, MenuFileName: str, BaseMenu: bool = None) -> AcadMenuGroup: ...
    def __iter__(self): ...



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadBlockReference(AcadEntity):
    def __init__(self, obj) -> None: ...
    EffectiveName: str
    HasAttributes: bool
    InsertionPoint: PyGePoint3d
    InsUnits: str
    InsUnitsFactor: float
    IsDynamicBlock: bool
    Name: bool
    Normal: PyGeVector3d
    Rotation: float
    XEffectiveScaleFactor: float
    XScaleFactor: float
    YEffectiveScaleFactor: float
    YScaleFactor: float
    ZEffectiveScaleFactor: float
    ZScaleFactor: float
    def ConvertToAnonymousBlock(self) -> None: ...
    def ConvertToStaticBlock(self, newBlockName: str) -> None: ...
    def Copy(self) -> AcadBlockReference: ...
    def Explode(self) -> vObjectArray: ...
    def GetAttributes(self) -> vObjectArray[AcadAttributeReference]: ...
    def GetConstantAttributes(self) -> vObjectArray[AcadAttribute]: ...
    def GetDynamicBlockProperties(self) -> vObjectArray[AcadDynamicBlockReferenceProperty]: ...
    def ResetBlock(self) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *

        
        
class AcadPreferencesOpenSave(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    AutoAudit: bool = proxy_property(bool,'AutoAudit',AccessMode.ReadWrite)
    AutoSaveInterval: int = proxy_property(int,'AutoSaveInterval',AccessMode.ReadWrite)
    CreateBackup: bool = proxy_property(bool,'CreateBackup',AccessMode.ReadWrite)
    DemandLoadARXApp: AcARXDemandLoad  = proxy_property('AcARXDemandLoad','DemandLoadARXApp',AccessMode.ReadWrite)
    FullCRCValidation: bool  = proxy_property(bool,'FullCRCValidation',AccessMode.ReadWrite)
    IncrementalSavePercent: int  = proxy_property(int,'IncrementalSavePercent',AccessMode.ReadWrite)
    LogFileOn: bool  = proxy_property(bool,'LogFileOn',AccessMode.ReadWrite)
    MRUNumber: int  = proxy_property(int,'MRUNumber',AccessMode.ReadOnly)
    ProxyImage: AcProxyImage = proxy_property('AcProxyImage','ProxyImage',AccessMode.ReadWrite)
    SaveAsType: AcSaveAsType  = proxy_property('AcSaveAsType','SaveAsType',AccessMode.ReadWrite)
    SavePreviewThumbnail: bool  = proxy_property(bool,'SavePreviewThumbnail',AccessMode.ReadWrite)
    ShowProxyDialogBox: bool  = proxy_property(bool,'ShowProxyDialogBox',AccessMode.ReadWrite)
    TempFileExtension: str  = proxy_property(str,'TempFileExtension',AccessMode.ReadWrite)
    XRefDemandLoad: AcXRefDemandLoad = proxy_property('AcXRefDemandLoad','XRefDemandLoad',AccessMode.ReadWrite)


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadRasterImage(AcadEntity):
    def __init__(self, obj) -> None: ...
    Brightness: int
    ClippingEnabled: bool
    Contrast: int
    Fade: int
    Height: float
    ImageFile: str
    ImageHeight: float
    ImageVisibility: bool
    ImageWidth: float
    Name: str
    Origin: PyGePoint3d
    Rotation: float
    ScaleFactor: float
    ShowRotation: bool
    Transparency: bool
    Width: float
    def ClipBoundary(self, PointsArray: PyGePoint2dArray) -> None: ...
    def Copy(self) -> AcadRasterImage: ...



from .VarType import *
from enum import IntEnum
from .Ac import AdeskDxfCode

class DxfGroupXDCode(IntEnum):
    """
    Перечисление кодов групп DXF, используемых для Extended Data (XData). [1000 - 1071]
    """
    # APP_NAME = AdeskDxfCode.kDxfRegAppName       # Строковое значение имени приложения (регистратора)
    # STRING = AdeskDxfCode.kDxfXdAsciiString         # Общее строковое значение XData
    # EXT_ASC_STRING = 1001 # Расширенная строка (для юникода)
    # CONTROL_STRING = 1002 # Строка управления (например, "{", "}")
    # LAYER_NAME = AdeskDxfCode.kDxfXdLayerName     # Имя слоя
    # DATABASE_HANDLE = 1005 # Дескриптор базы данных
    # POSITION_X = 1010     # X-координата позиции (3D точка, индексы 10, 20, 30 вместе)
    # POSITION_Y = 1020     # Y-координата позиции
    # POSITION_Z = 1030     # Z-координата позиции
    # DISTANCE = 1040       # Значение расстояния (с плавающей точкой)
    # SCALE = 1041          # Значение масштаба (с плавающей точкой)
    # INTEGER_16 = 1070     # 16-битное целое число (Short)
    # INTEGER_32 = 1071     # 32-битное целое число (Long/Int)
    kDxfXdAsciiString  = AdeskDxfCode.kDxfXdAsciiString
    kDxfRegAppName     = AdeskDxfCode.kDxfRegAppName
    kDxfXdControlString = AdeskDxfCode.kDxfXdControlString
    kDxfXdLayerName    = AdeskDxfCode.kDxfXdLayerName
    kDxfXdBinaryChunk  = AdeskDxfCode.kDxfXdBinaryChunk
    kDxfXdHandle       = AdeskDxfCode.kDxfXdHandle

    kDxfXdXCoord       = AdeskDxfCode.kDxfXdXCoord
    # kDxfXdYCoord       = AdeskDxfCode.kDxfXdYCoord
    # kDxfXdZCoord       = AdeskDxfCode.kDxfXdZCoord

    kDxfXdWorldXCoord  = AdeskDxfCode.kDxfXdWorldXCoord
    # kDxfXdWorldYCoord  = AdeskDxfCode.kDxfXdWorldYCoord
    # kDxfXdWorldZCoord  = AdeskDxfCode.kDxfXdWorldZCoord

    kDxfXdWorldXDisp   = AdeskDxfCode.kDxfXdWorldXDisp
    # kDxfXdWorldYDisp   = AdeskDxfCode.kDxfXdWorldYDisp
    # kDxfXdWorldZDisp   = AdeskDxfCode.kDxfXdWorldZDisp

    kDxfXdWorldXDir    = AdeskDxfCode.kDxfXdWorldXDir
    # kDxfXdWorldYDir    = AdeskDxfCode.kDxfXdWorldYDir
    # kDxfXdWorldZDir    = AdeskDxfCode.kDxfXdWorldZDir

    kDxfXdReal         = AdeskDxfCode.kDxfXdReal
    kDxfXdDist         = AdeskDxfCode.kDxfXdDist
    kDxfXdScale        = AdeskDxfCode.kDxfXdScale

    kDxfXdInteger16    = AdeskDxfCode.kDxfXdInteger16
    kDxfXdInteger32    = AdeskDxfCode.kDxfXdInteger32


class XDataManager():
    def __init__(self, regAppName: str = ''):
        self._codes = [DxfGroupXDCode.kDxfRegAppName]
        self._values = [regAppName]

    @property
    def RegAppName(self) -> str:
        return self._values[0]

    @RegAppName.setter
    def RegAppName(self, value: str) -> str:
        self._values[0] = value

    @property
    def xDataType(self) -> vShortArray:
        return vShortArray(self._codes)

    @property
    def xDataValue(self) -> vVariantArray:
        return vVariantArray(self._values)
    
    def add_data(self, group_code: DxfGroupXDCode|list[DxfGroupXDCode]|tuple[DxfGroupXDCode], value: int|str|float|list[int|str|float]|tuple[int|str|float]):
        self._codes += flatten_generic(group_code)
        self._values += flatten_generic(value)
        if len(self._codes) != len(self._values): raise IndexError(f'Каждому коду должно соответствовать значение. Кодов: {len(self._codes)}, значений: {len(self._values)}')

    def __getitem__(self, key):
        return self._codes[key], self._values[key]

    def __setitem__(self, key, value):
        if isinstance(value, (list, tuple)) and len(value)==2:
            self._codes[key] = DxfGroupXDCode(value[0])
            self._values[key] = value[1]
        else:
            raise ValueError(f'Не допустимое кол-во элементов или не верный тип данных: {value}')
            
    def __len__(self):
            return len(self._codes)

    def __iter__(self):
        for i in range(0,len(self._codes)):
            yield self._codes[i], self._values[i]

    def __repr__(self):
        return f'{self.__class__.__name__}({self.xDataType}, {self.xDataValue})'


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadPopupMenu import *

class AcadPopupMenus(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Count: int = proxy_property(int,'Count',AccessMode.ReadOnly)
    Parent: AppObject = proxy_property('AppObject','Parent',AccessMode.ReadWrite)

    def Add(self, Name: str) -> AcadPopupMenu: 
        return AcadPopupMenu(self._obj.Add(Name))

    def InsertMenuInMenuBar(self, MenuName: str, Index: int) -> None: 
        self._obj.InsertMenuInMenuBar(MenuName, Index)

    def Item(self, Index: int) -> AcadPopupMenu: 
        return AcadPopupMenu(self._obj.Item(Index))

    def RemoveMenuFromMenuBar(self, Index: int) -> None: 
        self._obj.RemoveMenuFromMenuBar(Index)

    def __iter__(self):
        for item in self._obj:
            obj = AcadPopupMenu(item)
            yield obj


from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadTableStyle(AcadObject):
    def __init__(self, obj) -> None: ...
    BitFlags: int
    Description: str
    FlowDirection: AcTableDirection
    HeaderSuppressed: bool
    HorzCellMargin: float
    Name: str
    NumCellStyles: int
    TemplateId: int
    TitleSuppressed: bool
    VertCellMargin: float
    def CreateCellStyle(self, StringCellStyle: str) -> None: ...
    def CreateCellStyleFromStyle(self, StringCellStyle: str, StringSourceCellStyle: str) -> None: ...
    def DeleteCellStyle(self, StringCellStyle: str) -> None: ...
    def EnableMergeAll(self, nRow: int, nCol: int, bEnable: bool) -> None: ...
    def GetAlignment(self, rowType: AcRowType) -> AcCellAlignment: ...
    def GetAlignment2(self, bstrCellStyle: str) -> AcCellAlignment: ...
    def GetBackgroundColor(self, rowType: AcRowType) -> AcadAcCmColor: ...
    def GetBackgroundColor2(self, bstrCellStyle: str) -> AcadAcCmColor: ...
    def GetBackgroundColorNone(self, rowType: AcRowType) -> bool: ...
    def GetCellClass(self, StringCellStyle: str) -> int: ...
    def GetCellStyles(self) -> Variant: ...
    def GetColor(self, rowType: AcRowType) -> AcadAcCmColor: ...
    def GetColor2(self, bstrCellStyle: str) -> AcadAcCmColor: ...
    def GetDataType(self, rowType: AcRowType) -> tuple: ...
    def GetDataType2(self, nRow: int, nCol: int, nContent: int) -> tuple: ...
    def GetFormat(self, rowType: AcRowType) -> str: ...
    def GetFormat2(self, StringCellStyle: str) -> str: ...
    def GetGridColor(self, gridLineType: AcGridLineType, rowType: AcRowType) -> AcadAcCmColor: ...
    def GetGridColor2(self, bstrCellStyle: str, gridLineType: AcGridLineType) -> AcadAcCmColor: ...
    def GetGridLineWeight(self, gridLineType: AcGridLineType, rowType: AcRowType) -> AcLineWeight: ...
    def GetGridLineWeight2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType) -> AcLineWeight: ...
    def GetGridVisibility(self, gridLineType: AcGridLineType, rowType: AcRowType) -> bool: ...
    def GetGridVisibility2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType) -> bool: ...
    def GetIsCellStyleInUse(self, pszCellStyle: str) -> bool: ...
    def GetIsMergeAllEnabled(self, StringCellStyle: str) -> bool: ...
    def GetRotation(self, StringCellStyle: str) -> float: ...
    def GetTextHeight(self, rowType: AcRowType) -> float: ...
    def GetTextHeight2(self, StringCellStyle: str) -> float: ...
    def GetTextStyle(self, rowType: AcRowType) -> str: ...
    def GetTextStyleId(self, bstrCellStyle: str) -> float: ...
    def GetUniqueCellStyleName(self, pszBaseName: str) -> str: ...
    def RenameCellStyle(self, StringOldName: str, StringNewName: str) -> None: ...
    def SetAlignment(self, rowTypes: AcRowType, cellAlignment: AcCellAlignment) -> None: ...
    def SetAlignment2(self, bstrCellStyle: str, cellAlignment: AcCellAlignment) -> None: ...
    def SetBackgroundColor(self, rowTypes: AcRowType, pColor: AcadAcCmColor) -> None: ...
    def SetBackgroundColor2(self, bstrCellStyle: str, color: AcadAcCmColor) -> None: ...
    def SetBackgroundColorNone(self, rowTypes: AcRowType, bValue: bool) -> None: ...
    def SetCellClass(self, StringCellStyle: str, cellClass: int) -> None: ...
    def SetColor(self, rowTypes: AcRowType, pColor: AcadAcCmColor) -> None: ...
    def SetColor2(self, bstrCellStyle: str, color: AcadAcCmColor) -> None: ...
    def SetDataType(self, rowTypes: AcRowType, nDataType: AcValueDataType, nUnitType: AcValueUnitType) -> None: ...
    def SetDataType2(self, nRow: int, nCol: int, nContent: int, dataType: AcValueDataType, unitType: AcValueUnitType) -> None: ...
    def SetFormat(self, rowTypes: AcRowType, pFormat: str) -> None: ...
    def SetFormat2(self, bstrCellStyle: str, bstrFormat: str) -> None: ...
    def SetGridColor(self, gridLineTypes: AcGridLineType, rowTypes: AcRowType, pColor: AcadAcCmColor) -> None: ...
    def SetGridColor2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType, pColor: AcadAcCmColor) -> None: ...
    def SetGridLineWeight(self, gridLineTypes: AcGridLineType, rowTypes: AcRowType, Lineweight: AcLineWeight) -> None: ...
    def SetGridLineWeight2(self, bstrCellStyle: str, gridLineType: AcGridLineType, Lineweight: AcLineWeight) -> None: ...
    def SetGridVisibility(self, gridLineTypes: AcGridLineType, rowTypes: AcRowType, bVisible: bool) -> None: ...
    def SetGridVisibility2(self, bstrCellStyle: str, gridLineType: AcGridLineType, bValue: bool) -> None: ...
    def SetRotation(self, bstrCellStyle: str, Rotation: float) -> None: ...
    def SetTemplateId(self, val: int, option: AcMergeCellStyleOption) -> None: ...
    def SetTextHeight(self, rowTypes: AcRowType, TextHeight: float) -> None: ...
    def SetTextHeight2(self, bstrCellStyle: str, Height: float) -> None: ...
    def SetTextStyle(self, rowTypes: AcRowType, bstrName: str) -> None: ...
    def SetTextStyleId(self, bstrCellStyle: str, val: int) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
# from ..AcadObject import *
from .AcadToolbar import *

class AcadToolbars(AppObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Count: int = proxy_property(int,'Count',AccessMode.ReadOnly)
    LargeButtons: bool = proxy_property(bool,'LargeButtons',AccessMode.ReadWrite)
    Parent: AppObject = proxy_property('AppObject','Parent',AccessMode.ReadWrite)

    def Add(self, Name: str) -> AcadToolbar:
        return AcadToolbar(self._obj.Add(Name))

    def Item(self, Index: int | str) -> AcadToolbar:
        return AcadToolbar(self._obj.Item(Index))

    def __iter__(self):
        for item in self._obj:
            obj = AcadToolbar(item)
            yield obj



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPreferencesFiles(AppObject):
    def __init__(self, obj) -> None: ...
    ActiveInvProject: str
    AltFontFile: str
    AltTabletMenuFile: str
    Application: AcadApplication
    AutoSavePath: str
    ColorBookPath: str
    ConfigFile: str
    CustomDictionary: str
    CustomIconPath: str
    DefaultInternetURL: str
    DriversPath: str
    EnterpriseMenuFile: str
    FontFileMap: str
    HelpFilePath: str
    LogFilePath: str
    MainDictionary: str
    MenuFile: str
    PageSetupOverridesTemplateFile: str
    PlotLogFilePath: str
    PostScriptPrologFile: str
    PrinterConfigPath: str
    PrinterDescPath: str
    PrinterStyleSheetPath: str
    PrintFile: str
    PrintSpoolerPath: str
    PrintSpoolExecutable: str
    QNewTemplateFile: str
    SupportPath: str
    TempFilePath: str
    TemplateDWGPath: str
    TempXRefPath: str
    TextEditor: str
    TextureMapPath: str
    ToolPalettePath: str
    WorkspacePath: str
    def GetProjectFilePath(self, ProjectName: str) -> Out[str]: ...
    def SetProjectFilePath(self, ProjectName: str, ProjectFilePath: str) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadDimension import AcadDimension



class AcadDimDiametric(AcadDimension):
    def __init__(self, obj) -> None: super().__init__(obj)

    AltRoundDistance: float = proxy_property(float,'AltRoundDistance',AccessMode.ReadWrite)
    AltSuppressLeadingZeros: bool = proxy_property(bool,'AltSuppressLeadingZeros',AccessMode.ReadWrite)
    AltSuppressTrailingZeros: bool = proxy_property(bool,'AltSuppressTrailingZeros',AccessMode.ReadWrite)
    AltSuppressZeroFeet: bool = proxy_property(bool,'AltSuppressZeroFeet',AccessMode.ReadWrite)
    AltSuppressZeroInches: bool = proxy_property(bool,'AltSuppressZeroInches',AccessMode.ReadWrite)
    AltTextPrefix: str = proxy_property(str,'AltTextPrefix',AccessMode.ReadWrite)
    AltTextSuffix: str = proxy_property(str,'AltTextSuffix',AccessMode.ReadWrite)
    AltTolerancePrecision: AcDimPrecision = proxy_property('AcDimPrecision','AltTolerancePrecision',AccessMode.ReadWrite)
    AltToleranceSuppressLeadingZeros: bool = proxy_property(bool,'AltToleranceSuppressLeadingZeros',AccessMode.ReadWrite)
    AltToleranceSuppressTrailingZeros: bool = proxy_property(bool,'AltToleranceSuppressTrailingZeros',AccessMode.ReadWrite)
    AltToleranceSuppressZeroFeet: bool = proxy_property(bool,'AltToleranceSuppressZeroFeet',AccessMode.ReadWrite)
    AltToleranceSuppressZeroInches: bool = proxy_property(bool,'AltToleranceSuppressZeroInches',AccessMode.ReadWrite)
    AltUnits: bool = proxy_property(bool,'AltUnits',AccessMode.ReadWrite)
    AltUnitsFormat: AcDimUnits = proxy_property('AcDimUnits','AltUnitsFormat',AccessMode.ReadWrite)
    AltUnitsPrecision: AcDimPrecision = proxy_property('AcDimPrecision','AltUnitsPrecision',AccessMode.ReadWrite)
    AltUnitsScale: float = proxy_property(float,'AltUnitsScale',AccessMode.ReadWrite)
    Arrowhead1Block: str = proxy_property(str,'Arrowhead1Block',AccessMode.ReadWrite)
    Arrowhead1Type: AcDimArrowheadType = proxy_property('AcDimArrowheadType','Arrowhead1Type',AccessMode.ReadWrite)
    Arrowhead2Block: str = proxy_property(str,'Arrowhead2Block',AccessMode.ReadWrite)
    Arrowhead2Type: AcDimArrowheadType = proxy_property('AcDimArrowheadType','Arrowhead2Type',AccessMode.ReadWrite)
    ArrowheadSize: int = proxy_property(int,'ArrowheadSize',AccessMode.ReadWrite)
    CenterMarkSize: float = proxy_property(float,'CenterMarkSize',AccessMode.ReadWrite)
    CenterType: AcDimCenterType = proxy_property('AcDimCenterType','CenterType',AccessMode.ReadWrite)
    DimConstrDesc: str = proxy_property(str,'DimConstrDesc',AccessMode.ReadWrite)
    DimConstrExpression: str = proxy_property(str,'DimConstrExpression',AccessMode.ReadWrite)
    DimConstrForm: bool = proxy_property(bool,'DimConstrForm',AccessMode.ReadWrite)
    DimConstrName: str = proxy_property(str,'DimConstrName',AccessMode.ReadWrite)
    DimConstrReference: bool = proxy_property(bool,'DimConstrReference',AccessMode.ReadWrite)
    DimConstrValue: str = proxy_property(str,'DimConstrValue',AccessMode.ReadWrite)
    DimensionLineColor: AcColor = proxy_property('AcColor','DimensionLineColor',AccessMode.ReadWrite)
    DimensionLinetype: str = proxy_property(str,'DimensionLinetype',AccessMode.ReadWrite)
    DimensionLineWeight: AcLineWeight = proxy_property('AcLineWeight','DimensionLineWeight',AccessMode.ReadWrite)
    DimLine1Suppress: bool = proxy_property(bool,'DimLine1Suppress',AccessMode.ReadWrite)
    DimLine2Suppress: bool = proxy_property(bool,'DimLine2Suppress',AccessMode.ReadWrite)
    Fit: AcDimFit = proxy_property('AcDimFit','Fit',AccessMode.ReadWrite)
    ForceLineInside: bool = proxy_property(bool,'ForceLineInside',AccessMode.ReadWrite)
    FractionFormat: AcDimFractionType = proxy_property('AcDimFractionType','FractionFormat',AccessMode.ReadWrite)
    LeaderLength: float = proxy_property(float,'LeaderLength',AccessMode.ReadWrite)
    LinearScaleFactor: float = proxy_property(float,'LinearScaleFactor',AccessMode.ReadWrite)
    Measurement: float = proxy_property(float,'Measurement',AccessMode.ReadOnly)
    PrimaryUnitsPrecision: AcDimPrecision = proxy_property('AcDimPrecision','PrimaryUnitsPrecision',AccessMode.ReadWrite)
    RoundDistance: float = proxy_property(float,'RoundDistance',AccessMode.ReadWrite)
    SuppressZeroFeet: bool = proxy_property(bool,'SuppressZeroFeet',AccessMode.ReadWrite)
    SuppressZeroInches: bool = proxy_property(bool,'SuppressZeroInches',AccessMode.ReadWrite)
    TextInside: bool = proxy_property(bool,'TextInside',AccessMode.ReadWrite)
    TextInsideAlign: bool = proxy_property(bool,'TextInsideAlign',AccessMode.ReadWrite)
    TextOutsideAlign: bool = proxy_property(bool,'TextOutsideAlign',AccessMode.ReadWrite)
    ToleranceSuppressZeroFeet: bool = proxy_property(bool,'ToleranceSuppressZeroFeet',AccessMode.ReadWrite)
    ToleranceSuppressZeroInches: bool = proxy_property(bool,'ToleranceSuppressZeroInches',AccessMode.ReadWrite)
    UnitsFormat: AcDimLUnits = proxy_property('AcDimLUnits','UnitsFormat',AccessMode.ReadWrite)

    def Copy(self) -> AcadDimDiametric:
        return AcadDimDiametric(self._obj.Copy())


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadLine(AcadEntity):
    def __init__(self, obj) -> None: ...
    Angle: float
    Delta: vDoubleArray
    EndPoint: PyGePoint3d
    Length: float
    Normal: PyGeVector3d
    StartPoint: PyGePoint3d
    Thickness: float
    def Offset(self, Distance: float) -> vObjectArray: ...
    def Copy(self) -> AcadLine: ...



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadDimension(AcadEntity):
    def __init__(self, obj) -> None: ...
    DecimalSeparator: str
    DimTxtDirection: bool
    Normal: PyGeVector3d
    Rotation: float
    ScaleFactor: float
    StyleName: str
    SuppressLeadingZeros: bool
    SuppressTrailingZeros: bool
    TextColor: AcColor
    TextFill: bool
    TextFillColor: AcColor
    TextGap: float
    TextHeight: float
    TextMovement: AcDimTextMovement
    TextOverride: str
    TextPosition: PyGePoint3d
    TextPrefix: str
    TextRotation: float
    TextStyle: str
    TextSuffix: str
    ToleranceDisplay: AcDimToleranceMethod
    ToleranceHeightScale: float
    ToleranceJustification: AcDimToleranceJustify
    ToleranceLowerLimit: float
    TolerancePrecision: AcDimPrecision
    ToleranceSuppressLeadingZeros: bool
    ToleranceSuppressTrailingZeros: bool
    ToleranceUpperLimit: float
    VerticalTextPosition: AcDimVerticalJustification
    def Copy(self) -> AcadDimension: ...



from .Base import *
from .Objects.AcadApplication import *
from .Proxy import *

class IAcadObject(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Document: AcadDocument
    Handle: str
    HasExtensionDictionary: bool
    ObjectName: str
    ObjectID: int
    OwnerID: int
    def GetExtensionDictionary(self) -> AcadDictionary: ...
    def GetXData(self, AppName: str = '') -> tuple: ...
    def SetXData(self, XDataType: vShortArray, XDataValue: vVariantArray) -> None: ...

class AcadObject(IAcadObject):
    def __init__(self, obj) -> None: ...
    def Delete(self) -> None: ...

class IAcadObjectCollection(IAcadObject):
    def __init__(self, obj) -> None: ...
    Count: int
    def Item(self, Index: int | str): ...
    def __iter__(self): ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadDatabase import AcadDatabase, IAcadDatabase
from .AcadUCS import AcadUCS



class AcadDocument(AcadDatabase):
    def __init__(self, obj) -> None: super().__init__(obj)

    Active: bool = proxy_property(bool,'Active',AccessMode.ReadOnly)
    ActiveDimStyle: AcadDimStyle = proxy_property('AcadDimStyle','ActiveDimStyle',AccessMode.ReadWrite)
    ActiveLayer: AcadLayer = proxy_property('AcadLayer','ActiveLayer',AccessMode.ReadWrite)
    ActiveLayout: AcadLayout = proxy_property('AcadLayout','ActiveLayout',AccessMode.ReadWrite)
    ActiveLinetype: AcadLineType = proxy_property('AcadLineType','ActiveLinetype',AccessMode.ReadWrite)
    ActiveMaterial: AcadMaterial = proxy_property('AcadMaterial','ActiveMaterial',AccessMode.ReadWrite)
    ActivePViewport: AcadPViewport = proxy_property('AcadPViewport','ActivePViewport',AccessMode.ReadWrite)
    ActiveSelectionSet: AcadSelectionSet = proxy_property('AcadSelectionSet','ActiveSelectionSet',AccessMode.ReadOnly)
    ActiveSpace: AcActiveSpace = proxy_property('AcActiveSpace','ActiveSpace',AccessMode.ReadWrite)
    ActiveTextStyle: AcadTextStyle = proxy_property('AcadTextStyle','ActiveTextStyle',AccessMode.ReadWrite)
    ActiveUCS: AcadUCS = proxy_property('AcadUCS','ActiveUCS',AccessMode.ReadWrite)
    ActiveViewport: AcadViewport = proxy_property('AcadViewport','ActiveViewport',AccessMode.ReadWrite)
    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Database: IAcadDatabase = proxy_property('IAcadDatabase','Database',AccessMode.ReadOnly)
    FullName: str = proxy_property(str,'FullName',AccessMode.ReadOnly)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    HWND: int = proxy_property(int,'HWND',AccessMode.ReadOnly)
    MSpace: bool = proxy_property(bool,'MSpace',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadOnly)
    ObjectSnapMode: bool = proxy_property(bool,'ObjectSnapMode',AccessMode.ReadWrite)
    Path: str = proxy_property(str,'Path',AccessMode.ReadOnly)
    PickfirstSelectionSet: AcadSelectionSet = proxy_property('AcadSelectionSet','PickfirstSelectionSet',AccessMode.ReadOnly)
    Plot: AcadPlot = proxy_property('AcadPlot','Plot',AccessMode.ReadOnly)
    ReadOnly: bool = proxy_property(bool,'ReadOnly',AccessMode.ReadOnly)
    Saved: bool = proxy_property(bool,'Saved',AccessMode.ReadOnly)
    SelectionSets: AcadSelectionSet = proxy_property('AcadSelectionSet','SelectionSets',AccessMode.ReadOnly)
    SummaryInfo: AcadSummaryInfo = proxy_property('AcadSummaryInfo','SummaryInfo',AccessMode.ReadOnly)
    Utility: AcadUtility = proxy_property('AcadUtility','Utility',AccessMode.ReadOnly)
    Width: float = proxy_property(float,'Width',AccessMode.ReadWrite)
    WindowState: AcWindowState = proxy_property('AcWindowState','WindowState',AccessMode.ReadWrite)
    WindowTitle: str = proxy_property(str,'WindowTitle',AccessMode.ReadOnly)

    def Activate(self) -> None: 
        self._obj.Activate()

    def AuditInfo(self, FixError: bool) -> None: 
        self._obj.AuditInfo(FixError)

    def Close(self, SaveChanges: bool = False, FileName: str = '') -> None: 
        self._obj.Close(SaveChanges, FileName)

    def EndUndoMark(self) -> None: 
        self._obj.EndUndoMark()

    def Export(self, FileName: str, Extension: str, SelectionSet: AcadSelectionSet) -> None: 
        self._obj.Export(FileName, Extension, SelectionSet)

    def GetVariable(self, Name: str) -> any: 
        return self._obj.GetVariable(Name)

    def Import(self, FileName: str, InsertionPoint: PyGePoint3d, ScaleFactor: float) -> AcadBlockReference | None: 
        ref = self._obj.Import(FileName, InsertionPoint(), ScaleFactor)
        return AcadBlockReference(ref) if ref else None

    def LoadShapeFile(self, FullName: str) -> None: 
        self._obj.LoadShapeFile(FullName)

    def New(self, TemplateFileName: str) -> AcadDocument: 
        return AcadDocument(self._obj.New(TemplateFileName))

    def Open(self, Name: str, ReadOnly: bool = False) -> AcadDocument: # Password: Variant = None
        return self._obj.Open(Name, ReadOnly)

    def PostCommand(self, Command: str) -> None: 
        self._obj.PostCommand(Command)

    def PurgeAll(self) -> None: 
        self._obj.PurgeAll()
        
    def Regen(self, WhichViewports: AcRegenType) -> None: 
        self._obj.Regen(WhichViewports)

    def Save(self) -> None: 
        self._obj.Save()

    def SaveAs(self, FileName: str, FileType: AcSaveAsType = AcSaveAsType.acNative, SecurityParams: AcadSecurityParams = None) -> None: 
        if SecurityParams is None:
            self._obj.SaveAs(FileName, FileType)
        else:
            raise Exception('AcadSecurityParams не поддерживается') 
            self._obj.SaveAs(FileName, FileType, SecurityParams)

    def SendCommand(self, Command: str) -> None: 
        self._obj.SendCommand(Command)

    def SetVariable(self, Name: str, Value: Variant) -> None: 
        self._obj.SetVariable(Name, Value())

    def StartUndoMark(self) -> None: 
        self._obj.StartUndoMark()

    def WBlock(self, FileName: str, SelectionSet: AcadSelectionSet) -> None: 
        self._obj.WBlock(FileName, SelectionSet)




from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *


class AcadMaterial(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Description: str = proxy_property(str,'Description',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)



from ..Base import *
from ..Proxy import *
from ..Objects.AcadApplication import AcadApplication as AcadApplication

class AcadMenuBar(AppObjectCollection):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Count: int
    Parent: AppObject



from simpleautocad import *
from colorama import Back as Back, Style as Style
from typing import Generator

OFFSET_DIR: PyGeVector3d
OFFSET_COUNT: int
OFFSET_DIST_X: int
OFFSET_DIST_Y: int
OFFSET_DIST_Z: int

def init_point(): ...
def acad_app() -> Generator[AutoCAD, None, None]: ...
def acad_active_document(acad_app: AutoCAD) -> Generator[AcadDocument, None, None]: ...
def acad_model_space(acad_active_document: AcadDocument) -> Generator[AcadModelSpace, None, None]: ...
def object_AcadCircle(acad_model_space: AcadModelSpace) -> Generator[AcadCircle, None, None]: ...
def object_Acad3DFace(acad_model_space: AcadModelSpace) -> Generator[Acad3DFace, None, None]: ...
def object_AcadPolygonMesh(acad_model_space: AcadModelSpace) -> Generator[AcadPolygonMesh, None, None]: ...
def object_Acad3DPolyline(acad_model_space: AcadModelSpace) -> Generator[Acad3DPolyline, None, None]: ...
def object_AddArc(acad_model_space: AcadModelSpace) -> Generator[AcadArc, None, None]: ...
def object_AcadAttribute(acad_model_space: AcadModelSpace) -> Generator[AcadAttribute, None, None]: ...
def object_AddMInsertBlock(acad_model_space: AcadModelSpace) -> Generator[AcadBlock, None, None]: ...



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPreferencesUser(AppObject):
    def __init__(self, obj) -> None: ...
    ADCInsertUnitsDefaultSource: AcInsertUnits
    ADCInsertUnitsDefaultTarget: AcInsertUnits
    Application: AcadApplication
    HyperlinkDisplayCursor: bool
    KeyboardAccelerator: AcKeyboardAccelerator
    KeyboardPriority: AcKeyboardPriority
    SCMCommandMode: AcDrawingAreaSCMCommand
    SCMDefaultMode: AcDrawingAreaSCMDefault
    SCMEditMode: AcDrawingAreaSCMEdit
    SCMTimeMode: bool
    SCMTimeValue: int
    ShortCutMenuDisplay: bool



from ..Base import *
from ..Proxy import *
from .AcadRasterImage import AcadRasterImage as AcadRasterImage

class AcadGeomapImage(AcadRasterImage):
    def __init__(self, obj) -> None: ...
    GeoImageBrightness: int
    GeoImageContrast: int
    GeoImageFade: int
    GeoImageHeight: float
    GeoImageWidth: float
    def Copy(self) -> AcadGeomapImage: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadDimension import AcadDimension



class AcadDimAligned(AcadDimension):
    def __init__(self, obj) -> None: super().__init__(obj)

    AltRoundDistance: float = proxy_property(float,'AltRoundDistance',AccessMode.ReadWrite)
    AltSubUnitsFactor: float = proxy_property(float,'AltSubUnitsFactor',AccessMode.ReadWrite)
    AltSubUnitsSuffix: str = proxy_property(str,'AltSubUnitsSuffix',AccessMode.ReadWrite)
    AltSuppressLeadingZeros: bool = proxy_property(bool,'AltSuppressLeadingZeros',AccessMode.ReadWrite)
    AltSuppressTrailingZeros: bool = proxy_property(bool,'AltSuppressTrailingZeros',AccessMode.ReadWrite)
    AltSuppressZeroFeet: bool = proxy_property(bool,'AltSuppressZeroFeet',AccessMode.ReadWrite)
    AltSuppressZeroInches: bool = proxy_property(bool,'AltSuppressZeroInches',AccessMode.ReadWrite)
    AltTextPrefix: str = proxy_property(str,'AltTextPrefix',AccessMode.ReadWrite)
    AltTextSuffix: str = proxy_property(str,'AltTextSuffix',AccessMode.ReadWrite)
    AltTolerancePrecision: AcDimPrecision = proxy_property('AcDimPrecision','AltTolerancePrecision',AccessMode.ReadWrite)
    AltToleranceSuppressLeadingZeros: bool = proxy_property(bool,'AltToleranceSuppressLeadingZeros',AccessMode.ReadWrite)
    AltToleranceSuppressTrailingZeros: bool = proxy_property(bool,'AltToleranceSuppressTrailingZeros',AccessMode.ReadWrite)
    AltToleranceSuppressZeroFeet: bool = proxy_property(bool,'AltToleranceSuppressZeroFeet',AccessMode.ReadWrite)
    AltToleranceSuppressZeroInches: bool = proxy_property(bool,'AltToleranceSuppressZeroInches',AccessMode.ReadWrite)
    AltUnits: bool = proxy_property(bool,'AltUnits',AccessMode.ReadWrite)
    AltUnitsFormat: AcDimUnits = proxy_property('AcDimUnits','AltUnitsFormat',AccessMode.ReadWrite)
    AltUnitsPrecision: AcDimPrecision = proxy_property('AcDimPrecision','AltUnitsPrecision',AccessMode.ReadWrite)
    AltUnitsScale: float = proxy_property(float,'AltUnitsScale',AccessMode.ReadWrite)
    Arrowhead1Block: str = proxy_property(str,'Arrowhead1Block',AccessMode.ReadWrite)
    Arrowhead1Type: AcDimArrowheadType = proxy_property('AcDimArrowheadType','Arrowhead1Type',AccessMode.ReadWrite)
    Arrowhead2Block: str = proxy_property(str,'Arrowhead2Block',AccessMode.ReadWrite)
    Arrowhead2Type: AcDimArrowheadType = proxy_property('AcDimArrowheadType','Arrowhead2Type',AccessMode.ReadWrite)
    ArrowheadSize: int = proxy_property(int,'ArrowheadSize',AccessMode.ReadWrite)
    DimConstrDesc: str = proxy_property(str,'DimConstrDesc',AccessMode.ReadWrite)
    DimConstrExpression: str = proxy_property(str,'DimConstrExpression',AccessMode.ReadWrite)
    DimConstrForm: bool = proxy_property(bool,'DimConstrForm',AccessMode.ReadWrite)
    DimConstrName: str = proxy_property(str,'DimConstrName',AccessMode.ReadWrite)
    DimConstrReference: bool = proxy_property(bool,'DimConstrReference',AccessMode.ReadWrite)
    DimConstrValue: str = proxy_property(str,'DimConstrValue',AccessMode.ReadWrite)
    DimensionLineColor: AcColor = proxy_property('AcColor','DimensionLineColor',AccessMode.ReadWrite)
    DimensionLinetype: str = proxy_property(str,'DimensionLinetype',AccessMode.ReadWrite)
    DimensionLineWeight: AcLineWeight = proxy_property('AcLineWeight','DimensionLineWeight',AccessMode.ReadWrite)
    DimLine1Suppress: bool = proxy_property(bool,'DimLine1Suppress',AccessMode.ReadWrite)
    DimLine2Suppress: bool = proxy_property(bool,'DimLine2Suppress',AccessMode.ReadWrite)
    DimLineInside: bool = proxy_property(bool,'DimLineInside',AccessMode.ReadWrite)
    ExtensionLineColor: AcColor = proxy_property('AcColor','ExtensionLineColor',AccessMode.ReadWrite)
    ExtensionLineExtend: float = proxy_property(float,'ExtensionLineExtend',AccessMode.ReadWrite)
    ExtensionLineOffset: float = proxy_property(float,'ExtensionLineOffset',AccessMode.ReadWrite)
    ExtensionLineWeight: AcLineWeight = proxy_property('AcLineWeight','ExtensionLineWeight',AccessMode.ReadWrite)
    ExtLine1Linetype: str = proxy_property(str,'ExtLine1Linetype',AccessMode.ReadWrite)
    ExtLine1Point: PyGePoint3d = proxy_property('PyGePoint3d','ExtLine1Point',AccessMode.ReadWrite)
    ExtLine1Suppress: bool = proxy_property(bool,'ExtLine1Suppress',AccessMode.ReadWrite)
    ExtLine2Point: PyGePoint3d = proxy_property('PyGePoint3d','ExtLine2EndPoint',AccessMode.ReadWrite)
    ExtLine2Linetype: str = proxy_property(str,'ExtLine2Linetype',AccessMode.ReadWrite)
    ExtLine2Suppress: bool = proxy_property(bool,'ExtLine2Suppress',AccessMode.ReadWrite)
    ExtLineFixedLen: float = proxy_property(float,'ExtLineFixedLen',AccessMode.ReadWrite)
    ExtLineFixedLenSuppress: bool = proxy_property(bool,'ExtLineFixedLenSuppress',AccessMode.ReadWrite)
    Fit: AcDimFit = proxy_property('AcDimFit','Fit',AccessMode.ReadWrite)
    ForceLineInside: bool = proxy_property(bool,'ForceLineInside',AccessMode.ReadWrite)
    FractionFormat: AcDimFractionType = proxy_property('AcDimFractionType','FractionFormat',AccessMode.ReadWrite)
    HorizontalTextPosition: AcDimHorizontalJustification = proxy_property('AcDimHorizontalJustification','HorizontalTextPosition',AccessMode.ReadWrite)
    LinearScaleFactor: float = proxy_property(float,'LinearScaleFactor',AccessMode.ReadWrite)
    Measurement: float = proxy_property(float,'Measurement',AccessMode.ReadOnly)
    PrimaryUnitsPrecision: AcDimPrecision = proxy_property('AcDimPrecision','PrimaryUnitsPrecision',AccessMode.ReadWrite)
    RoundDistance: float = proxy_property(float,'RoundDistance',AccessMode.ReadWrite)
    SubUnitsFactor: float = proxy_property(float,'SubUnitsFactor',AccessMode.ReadWrite)
    SubUnitsSuffix: str = proxy_property(str,'SubUnitsSuffix',AccessMode.ReadWrite)
    SuppressZeroFeet: bool = proxy_property(bool,'SuppressZeroFeet',AccessMode.ReadWrite)
    SuppressZeroInches: bool = proxy_property(bool,'SuppressZeroInches',AccessMode.ReadWrite)
    TextInside: bool = proxy_property(bool,'TextInside',AccessMode.ReadWrite)
    TextInsideAlign: bool = proxy_property(bool,'TextInsideAlign',AccessMode.ReadWrite)
    TextOutsideAlign: bool = proxy_property(bool,'TextOutsideAlign',AccessMode.ReadWrite)
    ToleranceSuppressZeroFeet: bool = proxy_property(bool,'ToleranceSuppressZeroFeet',AccessMode.ReadWrite)
    ToleranceSuppressZeroInches: bool = proxy_property(bool,'ToleranceSuppressZeroInches',AccessMode.ReadWrite)
    UnitsFormat: AcDimLUnits = proxy_property('AcDimLUnits','UnitsFormat',AccessMode.ReadWrite)

    def Copy(self) -> AcadDimAligned:
        return AcadDimAligned(self._obj.Copy())


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadShape(AcadEntity):
    def __init__(self, obj) -> None: ...
    InsertionPoint: PyGePoint3d
    Name: str
    Normal: PyGeVector3d
    ObliqueAngle: float
    Rotation: float
    ScaleFactor: float
    Thickness: float
    def Copy(self) -> AcadShape: ...



from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadToolbar import *

class AcadToolbarItem(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    CommandDisplayName: str
    Flyout: AcadToolbar
    HelpString: str
    Index: int
    Macro: str
    Name: str
    Parent: AppObject
    TagString: str
    Type: AcToolbarItemType
    def AttachToolbarToFlyout(self, MenuGroupName: str, ToolbarName: str) -> None: ...
    def Delete(self) -> None: ...
    def GetBitmaps(self) -> tuple[str]: ...
    def SetBitmaps(self, SmallIconName: str, LargeIconName: str) -> None: ...



from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadLayout import *

class AcadLayouts(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    def Add(self, InsertionPoint: PyGePoint3d, Name: str) -> AcadLayout: ...
    def Item(self, Index: int) -> AcadLayout: ...
    def __iter__(self): ...



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *
from .AcadMenuGroup import *



class AcadMenuGroups(AppObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: 'AcadApplication' = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Count: int = proxy_property(int,'Count',AccessMode.ReadOnly)
    Parent: AppObject = proxy_property('AppObject','Parent',AccessMode.ReadWrite)

    def Item(self, Index: int | str) -> AcadMenuGroup: 
        return AcadMenuGroup(self._obj.Item(Index))

    def Load(self, MenuFileName: str, BaseMenu: bool = None) -> AcadMenuGroup: 
        if BaseMenu:
            return AcadMenuGroup(self._obj.Load(MenuFileName, BaseMenu))
        return AcadMenuGroup(self._obj.Load(MenuFileName))

    def __iter__(self):
        for item in self._obj:
            obj = AcadMenuGroup(item)
            yield obj


from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPlotConfiguration(AcadObject):
    def __init__(self, obj) -> None: ...
    CanonicalMediaName: str
    CenterPlot: bool
    ConfigName: str
    ModelType: bool
    Name: str
    PaperUnits: AcPlotPaperUnits
    PlotHidden: bool
    PlotOrigin: PyGePoint2d
    PlotRotation: AcPlotRotation
    PlotType: AcPlotType
    PlotViewportBorders: bool
    PlotViewportsFirst: bool
    PlotWithLineweights: bool
    PlotWithPlotStyles: bool
    ScaleLineweights: bool
    ShowPlotStyles: bool
    StandardScale: AcPlotScale
    StyleSheet: str
    UseStandardScale: bool
    ViewToPlot: str
    def CopyFrom(self, SourceObject: AcadPlotConfiguration) -> None: ...
    def GetCanonicalMediaNames(self) -> tuple[str]: ...
    def GetCustomScale(self) -> vDoubleArray: ...
    def GetLocaleMediaName(self, Name: str) -> str: ...
    def GetPaperMargins(self) -> vDoubleArray: ...
    def GetPaperSize(self) -> tuple: ...
    def GetPlotDeviceNames(self) -> tuple[str]: ...
    def GetPlotStyleTableNames(self) -> tuple[str]: ...
    def GetWindowToPlot(self) -> PyGePoint2dArray: ...
    def RefreshPlotDeviceInfo(self) -> None: ...
    def SetCustomScale(self, Numerator: float, Denominator: float) -> None: ...
    def SetWindowToPlot(self, LowerLeft: PyGePoint2d, UpperRight: PyGePoint2d) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadPoint(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Coordinates: PyGePoint3d = proxy_property('PyGePoint3d','Coordinates',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    Thickness: float = proxy_property(float,'Thickness',AccessMode.ReadWrite)

    def Copy(self) -> AcadPoint:
        return AcadPoint(self._obj.Copy())


from simpleautocad import *
import math
from colorama import init, Fore, Back, Style
init(autoreset=True) 

def prop_test(obj, name:str, assertion:type, access:AccessMode, write_value = None):
    print(f'Свойство: {Fore.CYAN}{name}{Fore.WHITE}, режим доступа: {Fore.CYAN}{access.name}{Fore.WHITE}')
    if access is AccessMode.ReadOnly:
        read_prop = getattr(obj, name)
        assert read_prop.__class__ is assertion
        print(f'\tчтение: {Fore.CYAN}{read_prop.__class__.__name__}{Fore.WHITE}({read_prop})')
    if access is AccessMode.ReadWrite:
        if write_value is not None:
            setattr(obj, name, write_value)
            print(f'\tзапись: {Fore.CYAN}{write_value.__class__.__name__}{Fore.WHITE}({write_value})')
        read_prop = getattr(obj, name)
        assert read_prop.__class__ is assertion
        print(f'\tчтение: {Fore.CYAN}{read_prop.__class__.__name__}{Fore.WHITE}({read_prop})')
        setattr(obj, name, read_prop)
        print(f'\tзапись: {Fore.CYAN}{read_prop.__class__.__name__}{Fore.WHITE}({read_prop})')
    if access is AccessMode.WriteOnly:
        setattr(obj, name, write_value)
        print(f'\tзапись: {Fore.CYAN}{write_value.__class__.__name__}{Fore.WHITE}({write_value})')
    if access is AccessMode.DenyFromAll:
        pass

def method_test(obj, name:str, ret_type = any, **kwargs):
    method = getattr(obj, name, None)
    result = None
    if callable(method):
        result = method(**kwargs)
        if ret_type is None or ret_type is any: pass
        else: assert ret_type is result.__class__
    else:
        raise AttributeError(f"Метод '{name}' не найден или не может быть вызван для объекта {type(obj).__name__}")
    items_str:str = ''
    for key, value in kwargs.items():
        items_str += f"{key} = {Fore.CYAN}{value.__class__.__name__}{Fore.WHITE}({value}); "
    if items_str: items_str = items_str[:-2]
    print(f'Метод: {Fore.CYAN}{name}{Fore.WHITE}({items_str})\n\tвернул: {Fore.CYAN}{result if result is None else result.__class__.__name__}{Fore.WHITE}({result})')
    return result

def test_AcadApplication(acad_app:AcadApplication):
    print(f"\nТестирование AcadApplication")
    print('-'*50)
    prop_test(acad_app, 'ActiveDocument', AcadDocument, AccessMode.ReadOnly)
    prop_test(acad_app, 'Application', AcadApplication, AccessMode.ReadOnly)
    prop_test(acad_app, 'Caption', str, AccessMode.ReadOnly)
    prop_test(acad_app, 'Documents', AcadDocuments, AccessMode.ReadOnly)
    prop_test(acad_app, 'FullName', str, AccessMode.ReadOnly)
    prop_test(acad_app, 'Height', float, AccessMode.ReadWrite)
    prop_test(acad_app, 'HWND', int, AccessMode.ReadOnly)
    prop_test(acad_app, 'LocaleId', int, AccessMode.ReadOnly)
    prop_test(acad_app, 'MenuBar', AcadMenuBar, AccessMode.ReadOnly)
    prop_test(acad_app, 'MenuGroups', AcadMenuGroups, AccessMode.ReadOnly)
    prop_test(acad_app, 'Name', str, AccessMode.ReadOnly)
    prop_test(acad_app, 'Path', str, AccessMode.ReadOnly)
    prop_test(acad_app, 'Preferences', AcadPreferences, AccessMode.ReadOnly)
    prop_test(acad_app, 'VBE', AppObject, AccessMode.ReadOnly)
    prop_test(acad_app, 'Version', str, AccessMode.ReadOnly)
    prop_test(acad_app, 'Visible', bool, AccessMode.ReadWrite)
    prop_test(acad_app, 'Width', float, AccessMode.ReadWrite)
    prop_test(acad_app, 'WindowLeft', int, AccessMode.ReadWrite)
    prop_test(acad_app, 'WindowState', int, AccessMode.ReadWrite)
    prop_test(acad_app, 'WindowTop', int, AccessMode.ReadWrite)

    
    for vp in acad_app.ActiveDocument.Viewports:
        method_test(acad_app,'StatusID',bool,VportObj=vp)

    method_test(acad_app,'Eval',None,Expression='n = 1 + 1')
    method_test(acad_app,'GetAcadState',AcadState)
    method_test(acad_app,'GetInterfaceObject',AppObject,ProgID=GetProgID(acad_app,'AcCmColor'))
    method_test(acad_app,'GetInterfaceObject',AppObject,ProgID=GetProgID(acad_app,'AcadLayerStateManager'))
    method_test(acad_app,'GetInterfaceObject',AppObject,ProgID=GetProgID(acad_app,'SecurityParams'))
    method_test(acad_app,'ListARX',vStringArray)
    method_test(acad_app,'Update',None)
    method_test(acad_app,'ZoomAll',None)
    method_test(acad_app,'ZoomCenter',None, Center=PyGePoint3d(),Magnify=10.5)
    method_test(acad_app,'ZoomExtents',None)
    # method_test(acad_app,'ZoomPickWindow',None)
    method_test(acad_app,'ZoomPrevious',None)
    method_test(acad_app,'ZoomScaled',None, Scale=0.5,ScaleType=AcZoomScaleType.acZoomScaledAbsolute)
    method_test(acad_app,'ZoomWindow',None, LowerLeft=PyGePoint3d(),UpperRight=PyGePoint3d(100,100))



def test_AcadDocument(acad_active_document:AcadDocument):
    print(f"\nТестирование AcadDocument")
    print('-'*50)
    doc = acad_active_document.Application.Documents.Add('New test doc')
    prop_test(doc, 'Active', bool, AccessMode.ReadOnly)
    prop_test(doc, 'ActiveDimStyle', AcadDimStyle, AccessMode.ReadWrite)
    prop_test(doc, 'ActiveLayer', AcadLayer, AccessMode.ReadWrite)
    prop_test(doc, 'ActiveLayout', AcadLayout, AccessMode.ReadWrite)
    prop_test(doc, 'ActiveLinetype', AcadLineType, AccessMode.ReadWrite)
    prop_test(doc, 'ActiveMaterial', AcadMaterial, AccessMode.ReadWrite)
    # prop_test(acad_active_document, 'ActivePViewport', AcadPViewport, AccessMode.ReadWrite)
    prop_test(doc, 'ActiveSelectionSet', AcadSelectionSet, AccessMode.ReadOnly)
    prop_test(doc, 'ActiveSpace', AcActiveSpace, AccessMode.ReadWrite)
    prop_test(doc, 'ActiveTextStyle', AcadTextStyle, AccessMode.ReadWrite)
    ucs = doc.UserCoordinateSystems.Add(PyGePoint3d(10,10,10),PyGePoint3d(10,0,10),PyGePoint3d(0,10,10),'TestUCS')
    prop_test(doc, 'ActiveUCS', AcadUCS, AccessMode.ReadWrite,ucs)
    vp = doc.Viewports.Add('TestViewport')
    vp.Direction = PyGeVector3d(1,1,1)
    prop_test(doc, 'ActiveViewport', AcadViewport, AccessMode.ReadWrite, vp)
    prop_test(doc, 'Application', AcadApplication, AccessMode.ReadOnly)
    prop_test(doc, 'Database', IAcadDatabase, AccessMode.ReadOnly)
    prop_test(doc, 'FullName', str, AccessMode.ReadOnly)
    prop_test(doc, 'Height', float, AccessMode.ReadWrite)
    prop_test(doc, 'HWND', int, AccessMode.ReadOnly)
    # prop_test(acad_active_document, 'MSpace', bool, AccessMode.ReadWrite)
    prop_test(doc, 'Name', str, AccessMode.ReadOnly)
    prop_test(doc, 'ObjectSnapMode', bool, AccessMode.ReadWrite)
    prop_test(doc, 'Saved', bool, AccessMode.ReadOnly)
    prop_test(doc, 'SelectionSets', AcadSelectionSet, AccessMode.ReadOnly)
    prop_test(doc, 'SummaryInfo', AcadSummaryInfo, AccessMode.ReadOnly)
    prop_test(doc, 'Utility', AcadUtility, AccessMode.ReadOnly)
    prop_test(doc, 'Width', float, AccessMode.ReadWrite)
    prop_test(doc, 'WindowState', AcWindowState, AccessMode.ReadWrite)
    prop_test(doc, 'WindowTitle', str, AccessMode.ReadOnly)

    method_test(doc,'Activate', None)
    method_test(doc,'AuditInfo', None, FixError=False)
    method_test(doc,'Close', None)
    # method_test(acad_active_document,'GetVariable', any, Name='CPLOTSTYLE')
    # method_test(acad_active_document.Application.Documents,'Open', AcadDocument, Name='acad.dwt')

def test_transform(acad_app:AutoCAD, acad_active_document:AcadDocument, acad_model_space:AcadModelSpace):
    center = PyGePoint3d()
    size = 10
    acad_app.ZoomCenter(center,10.0)
    col_h = acad_app.uGetAcadAcCmColor()
    col_h.SetRGB(0,0,255)    
    col_m = acad_app.uGetAcadAcCmColor()
    col_m.SetRGB(0,255,0)
    col_s = acad_app.uGetAcadAcCmColor()
    col_s.SetRGB(255,0,0)
    ang_1_60 = -math.radians(360/60)

    pt1 = PyGePoint3d(center.x,center.y+size - size/10)
    pt2 = PyGePoint3d(center.x,center.y+size)
    line = acad_model_space.AddLine(pt1,pt2)
    hinspt = PyGePoint3d(pt1.x,pt1.y - size/6)
    hour = acad_model_space.AddMText(hinspt, size/6, '12')
    hour.Height = size/8
    hour.AttachmentPoint = AcAttachmentPoint.acAttachmentPointMiddleCenter
    hour.InsertionPoint = hinspt
    
    h_sec = acad_model_space.AddLine(PyGePoint3d(center.x,center.y - size/6),PyGePoint3d(center.x,center.y+size-size/10-size/40))
    h_sec.TrueColor = col_s
    h_sec.Lineweight = AcLineWeight.acLnWt030
    h_min = acad_model_space.AddLine(PyGePoint3d(center.x,center.y - size/6),PyGePoint3d(center.x,center.y+size-size/10-size/8))
    h_min.TrueColor = col_m
    h_min.Lineweight = AcLineWeight.acLnWt030
    h_hour = acad_model_space.AddLine(PyGePoint3d(center.x,center.y - size/6),PyGePoint3d(center.x,center.y+size-size/10-size/4))
    h_hour.TrueColor = col_h
    h_hour.Lineweight = AcLineWeight.acLnWt030
    for i in range(1,60):
        ang = ang_1_60*i
        line2 = line.Copy()
        line2.Rotate(center,ang)
        if not (i%5):
            spt = line2.StartPoint
            ept = line2.EndPoint
            line2.ScaleEntity(ept,1.5)
            line2.Lineweight = AcLineWeight.acLnWt050

            mat = PyGeMatrix3d.rotation(ang,PyGeVector3d.kZaxis,center)
            h2 = hour.Copy()
            h2.TextString = str(int(i/5))
            h2.TransformBy(mat)
            h2.Rotate(h2.InsertionPoint,-ang)


    line.ScaleEntity(line.EndPoint,1.5)
    line.Lineweight = AcLineWeight.acLnWt050
    acad_active_document.Regen(AcRegenType.acActiveViewport)
    from datetime import datetime
    mytime = datetime.now()
    my_h = mytime.hour
    my_m = mytime.minute
    my_s = mytime.second
    h_sec.Rotate(center,ang_1_60*my_s)
    h_min.Rotate(center,ang_1_60*my_m)
    h_hour.Rotate(center,ang_1_60*my_h*5+ang_1_60*(my_m // 12))
    seconds = 0
    while True:
        mytime = datetime.now()
        update = False
        if my_m != mytime.minute:
            h_min.Rotate(center,ang_1_60)
            my_m = mytime.minute
            if not (my_m % 12):
                h_hour.Rotate(center,ang_1_60)
            update = True
        if my_s != mytime.second:
            h_sec.Rotate(center,ang_1_60)
            my_s = mytime.second
            update = True
            seconds += 1
        if update:
            acad_active_document.Regen(AcRegenType.acActiveViewport)
            print(f"{my_h}:{my_m}:{my_s}")
        if seconds >= 3: break

def test_create(object_AcadCircle,
                object_Acad3DFace,
                object_AcadPolygonMesh,
                object_Acad3DPolyline,
                object_AddArc,
                object_AcadAttribute,
                object_AddMInsertBlock,
                ):
    objects = (object_AcadCircle,
                object_Acad3DFace,
                object_AcadPolygonMesh,
                object_Acad3DPolyline,
                object_AddArc,
                object_AcadAttribute,
                object_AddMInsertBlock
                )


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadSurface import AcadSurface



class AcadPlaneSurface(AcadSurface):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Copy(self) -> AcadPlaneSurface:
        return AcadPlaneSurface(self._obj.Copy())